@@ -4,7 +4,7 @@
 */
 
 /*
-** $Id: cgi.c,v 1.22 2001/10/26 04:11:33 mrsam Exp $
+** $Id: cgi.c,v 1.28 2004/05/23 14:28:24 mrsam Exp $
 */
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -110,6 +110,8 @@ int	c;
 struct cgi_arglist *argp;
 
 #if HAVE_LIBFCGI
+	cgihttpscriptptr_init();
+	cgihttpsscriptptr_init();
 	hascgiformfd=0;
 	rfc2045p=0;
 	if(cgi_args) {
@@ -154,7 +156,7 @@ struct cgi_arglist *argp;
 			if (cl > CGIMAXFORMDATAARG)
 			{
 				printf("Content-Type: text/html\n\n");
-				printf("<HTML><BODY><H1>Attachment size exceeds limit set by system administrator</H1></BODY></HTML>\n");
+				printf("<html><body><h1>Attachment size exceeds limit set by system administrator</h1></body></html>\n");
 				fake_exit(1);
 			}
 			cgi_formdata(cl);
@@ -170,7 +172,7 @@ struct cgi_arglist *argp;
 		if (cl > CGIMAXARG)
 		{
 			printf("Content-Type: text/html\n\n");
-			printf("<HTML><BODY><H1>Message size exceeds limit set by system administrator</H1></BODY></HTML>\n");
+			printf("<html><body><h1>Message size exceeds limit set by system administrator</h1></body></html>\n");
 			fake_exit(1);
 		}
 	cgi_args=malloc(cl+1);	/* Extra insurance */
@@ -327,6 +329,8 @@ static char *nybble(char *p, int *n)
 		(*n) = (*n) * 16 + (*p++ - '0');
 	else if ( *p >= 'A' && *p <= 'F')
 		(*n) = (*n) * 16 + (*p++ - 'A' + 10);
+	else if ( *p >= 'a' && *p <= 'f')
+		(*n) = (*n) * 16 + (*p++ - 'a' + 10);
 	return (p);
 }
 
@@ -358,9 +362,6 @@ int	c;
 		q=nybble(q, &c);
 		q=nybble(q, &c);
 
-		if ((char)c == (char)0xA0)	c=' ';
-			/* See above */
-
 		if (c && c != '\r')
 			/* Ignore CRs we get in TEXTAREAS */
 			*p++=c;
@@ -396,7 +397,7 @@ struct cgi_arglist *argp;
 
 /* multipart/formdata decoding */
 
-static const char *disposition_name, *disposition_filename;
+static char *disposition_name=NULL, *disposition_filename=NULL;
 
 static char *formargbuf;
 static char *formargptr;
@@ -410,7 +411,6 @@ static int save_formdata(const char *p, size_t l, void *miscptr)
 
 static void cgiformdecode(struct rfc2045 *p, struct rfc2045id *a, void *b)
 {
-const char *disposition;
 off_t start_pos, end_pos, start_body;
 char	buf[512];
 int	n;
@@ -419,13 +419,26 @@ off_t	dummy;
 	a=a;
 	b=b;
 
-	rfc2045_dispositioninfo(p, &disposition, &disposition_name,
-		&disposition_filename);
+	if (disposition_name)
+		free(disposition_name);
+	if (disposition_filename)
+		free(disposition_filename);
 
-	if (!disposition || strcmp(disposition, "form-data"))	return;
-	if (!disposition_name || !*disposition_name)	return;
+	if (rfc2231_udecodeDisposition(p, "name", NULL, &disposition_name) < 0
+	    ||
+	    rfc2231_udecodeDisposition(p, "filename", NULL,
+				       &disposition_filename) < 0)
+	{
+		disposition_name=disposition_filename=NULL;
+		enomem();
+	}
 
-	if (!disposition_filename || !*disposition_filename)
+	if (!p->content_disposition
+	    || strcmp(p->content_disposition, "form-data"))	return;
+
+	if (!*disposition_name)	return;
+
+	if (!*disposition_filename)
 	{
 		rfc2045_mimepos(p, &start_pos, &end_pos, &start_body,
 			&dummy, &dummy);
@@ -548,6 +561,7 @@ char	buf[BUFSIZ];
 	lseek(cgiformfd, 0L, SEEK_SET);
 	while ((n=read(cgiformfd, buf, sizeof(buf))) > 0)
 		rfc2045_parse(rfc2045p, buf, n);
+	rfc2045_parse_partial(rfc2045p);
 	rfc2045_decode(rfc2045p, &cgiformdecode, 0);
 
 }
@@ -563,7 +577,6 @@ struct cgigetfileinfo {
 
 static void cgifiledecode(struct rfc2045 *p, struct rfc2045id *a, void *b)
 {
-const char *disposition;
 off_t start_pos, end_pos, start_body;
 char	buf[512];
 int	n;
@@ -575,14 +588,27 @@ off_t	dummy;
 
 	if (c->filenum == 0)	return;	/* Already retrieved this one. */
 
-	rfc2045_dispositioninfo(p, &disposition, &disposition_name,
-		&disposition_filename);
+	if (disposition_name)
+		free(disposition_name);
+	if (disposition_filename)
+		free(disposition_filename);
 
-	if (!disposition || strcmp(disposition, "form-data"))	return;
+	if (rfc2231_udecodeDisposition(p, "name", NULL, &disposition_name) < 0
+	    ||
+	    rfc2231_udecodeDisposition(p, "filename", NULL,
+				       &disposition_filename) < 0)
+	{
+		disposition_name=disposition_filename=NULL;
+		enomem();
+	}
 
-	if (!disposition_name || !*disposition_name)	return;
+	if (!p->content_disposition
+	    || strcmp(p->content_disposition, "form-data"))	return;
+
+	if (!*disposition_name)	return;
+
+	if (!*disposition_filename)	return;
 
-	if (!disposition_filename || !*disposition_filename)	return;
 	rfc2045_mimepos(p, &start_pos, &end_pos, &start_body,
 			&dummy, &dummy);
 
