@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -12,10 +12,12 @@
 #include	"moduledel.h"
 #include	"maxlongsize.h"
 #include	"comsubmitclient.h"
-#include	"rfc2045.h"
-#include	"rfc2045charset.h"
-#include	"rfc822.h"
-#include	"auth.h"
+#include	"comfax.h"
+#include	"rfc2045/rfc2045.h"
+#include	"rfc2045/rfc2045charset.h"
+#include	"rfc822/rfc822.h"
+#include	"numlib/numlib.h"
+#include	"authlib/auth.h"
 #include	<stdlib.h>
 #include	<string.h>
 #if	HAVE_UNISTD_H
@@ -114,7 +116,7 @@ struct moduledel *p;
 	if (chdir(getenv("COURIER_HOME")))
 		clog_msg_errno();
 
-	authchangeuidgid(MAILUID, MAILGID);
+	libmail_changeuidgid(MAILUID, MAILGID);
 
 	dsnfrom=config_dsnfrom();
 	dsnlimit=config_dsnlimit();
@@ -668,6 +670,18 @@ struct dsndiagnosticinfo *i=(struct dsndiagnosticinfo *)q;
 	}
 }
 
+static int isfax(const char *p)
+{
+	int dummy;
+
+	if ((p=strrchr(p, '@')) == NULL)
+		return (0);
+
+	if (comgetfaxopts(p+1, &dummy) == 0)
+		return (1);
+	return (0);
+}
+
 static int dodsn(struct ctlfile *ctf, FILE *fp, const char *sender,
 		int dodelayed)
 {
@@ -944,7 +958,10 @@ const	char *from_time;
 		}
 
 		if (dodelayed &&
-			(j=ctlfile_searchfirst(ctf, COMCTLFILE_EXPIRES)) >= 0)
+			(j=ctlfile_searchfirst(ctf,
+					       isfax(ctf->receipients[i]) ?
+					       COMCTLFILE_FAXEXPIRES:
+					       COMCTLFILE_EXPIRES)) >= 0)
 			fprintf(fp, "Will-Retry-Until: %s\n",
 				rfc822_mkdate(strtotime(ctf->lines[j]+1)));
 	}
