@@ -38,7 +38,7 @@
 #include	"numlib/numlib.h"
 #include	"tcpd/spipe.h"
 #include	"tcpd/tlsclient.h"
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.33 2002/02/21 00:37:01 mrsam Exp $";
+static const char rcsid[]="$Id: courieresmtpd.c,v 1.44 2004/04/24 19:56:19 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
@@ -118,7 +118,8 @@ const char *p;
 	} while (*banner);
 }
 
-static void ehlo(const char *heloname, int hasauth, int hastls, int tls)
+static void ehlo(const char *heloname, int hasauth, int hastls, int tls,
+		 int extended)
 {
 static const char e[]=
 COURIER_EXTENSIONS
@@ -131,11 +132,29 @@ EHLO_SECURITY_EXTENSION
 "250 DSN\r\n";
 const char *me=config_me();
 
+	helobuf[0]=0;
+	strncat(helobuf, heloname, sizeof(helobuf)-1);
+
+	{
+		char *p;
+
+		for (p=helobuf; *p; p++)
+			if (isspace((int)(unsigned char)*p) ||
+			    (int)(unsigned char)*p < ' ')
+				*p='_';
+	}
+
+	if (!extended)
+	{
+		addiovec("250 ", 4);
+		addiovec(me, strlen(me));
+		addiovec(" Ok.\r\n", 6);
+		return;
+	}
+
 	addiovec("250-", 4);
 	addiovec(me, strlen(me));
 	addiovec(" Ok.\r\n", 6);
-	helobuf[0]=0;
-	strncat(helobuf, heloname, sizeof(helobuf)-1);
 	if (hasauth)
 	{
 		const char *p;
@@ -477,7 +496,7 @@ int	rc;
 #if HAVE_STRNCASECMP
 			strncasecmp(r, "EXDATA", 6)
 #else
-			strnicmp(r, "VERP", 6)
+			strnicmp(r, "EXDATA", 6)
 #endif
 			== 0)
 		{
@@ -573,7 +592,13 @@ int	rc;
 	if (rc)
 	{
 		iovflush();
-		sleep(5);
+		if (rc == -5)
+		{
+			sleep(teergrube);
+			teergrube *= 2;
+			if (teergrube > MAX_TEERGRUBE)
+				teergrube=MAX_TEERGRUBE;
+		}
 	}
 	return (rc);
 }
@@ -599,7 +624,60 @@ const char *q=skipaddress(&p);
 	return (-1);
 }
 
+static int dorcptto2(const char *p, const char *q);
+static int rcpttolocal(const char *p, const char *r, const char *q);
+
 static int dorcptto(const char *p, const char *q)
+{
+	const char *r;
+	const char *tcp=getenv("TCPLOCALIP");
+
+	/* foo@[our.ip.address] -> foo@default */
+
+	for (r=p; r != q; r++)
+	{
+		if (*r == '@' && r[1] == '[' && tcp)
+		{
+
+			if (strncasecmp(r+2, tcp, strlen(tcp)) == 0 &&
+			    strncasecmp(r+2+strlen(tcp), "]>", 2) == 0)
+			{
+				return rcpttolocal(p, r+1, q);
+			}
+
+			if (strncmp(tcp, "::ffff:", 7) == 0)
+			{
+				tcp += 7;
+				if (strncasecmp(r+2, tcp, strlen(tcp)) == 0 &&
+				    strncasecmp(r+2+strlen(tcp), "]>", 2) == 0)
+				{
+					return rcpttolocal(p, r+1, q);
+				}
+			}
+		}
+	}
+
+	return dorcptto2(p, q);
+}
+
+static int rcpttolocal(const char *p, const char *r, const char *q)
+{
+	const char *d=config_defaultdomain();
+	char *buf=courier_malloc(r-p + strlen(d)+strlen(q)+1);
+	int rc;
+
+	memcpy(buf, p, r-p);
+	strcpy(buf+(r-p), d);
+
+	p=buf+strlen(buf);
+	strcat(buf, q);
+
+	rc=dorcptto2(buf, p);
+	free(buf);
+	return rc;
+}
+
+static int dorcptto2(const char *p, const char *q)
 {
 int	notifyn=0,notifyf=0,notifys=0,notifyd=0;
 const char *orcpt=0;
@@ -691,10 +769,13 @@ int	rc;
 	if (rc)
 	{
 		iovflush();
-		sleep(teergrube);
-		teergrube *= 2;
-		if (teergrube > MAX_TEERGRUBE)
-			teergrube=MAX_TEERGRUBE;
+		if (rc == -5)
+		{
+			sleep(teergrube);
+			teergrube *= 2;
+			if (teergrube > MAX_TEERGRUBE)
+				teergrube=MAX_TEERGRUBE;
+		}
 
 	}
 	return (rc);
@@ -874,9 +955,17 @@ int	authenticated=0;
 
 		if ((p=getenv("ESMTP_TLS")) && atoi(p))
 		{
+			const char *p=getenv("TLS_CONNECTED_PROTOCOL");
+
 			tls=1;
 			/* This is the smtps service, and we don't need to
 			 * initialize the STARTTLS feature. */
+
+			if (p && *p)
+			{
+				strcpy(tlsbuf, "SSL: ");
+				strncat(tlsbuf, p, sizeof(tlsbuf)-6);
+			}
 		}
 		else
 		{
@@ -899,6 +988,7 @@ int	authenticated=0;
 		}
 	}
 
+
 	{
 		const char *p=getenv("AUTH_REQUIRED");
 
@@ -942,6 +1032,16 @@ int	authenticated=0;
 	char	*p;
 
 		input_line=line=iovreadline();
+
+		/* Optionally log the esmtp dialog */
+		if ((p=getenv("ESMTP_LOG_DIALOG")) && *p == '1')
+		{
+			clog_msg_start_info();
+			clog_msg_str(input_line);
+			clog_msg_str("\n");
+			clog_msg_send();
+		}
+
 		for (p=line; *p && *p != ':'; p++)
 		{
 			if (*p == ' ' && strncmp(line, "MAIL", 4) &&
@@ -956,7 +1056,7 @@ int	authenticated=0;
 			strncmp(line, "HELO ", 5) == 0) &&
 			line[5])
 		{
-			ehlo(line+5, argc > 1, starttls, tls);
+			ehlo(line+5, argc > 1, starttls, tls, line[0] == 'E');
 			iovflush();
 			cancelsubmit();
 			startsubmit(tls);
@@ -976,16 +1076,16 @@ int	authenticated=0;
 				addiovec_error(NULL, "540 TLS not available.");
 				continue;
 			}
-			if (s_pipe(pipefd))
+			if (libmail_streampipe(pipefd))
 			{
-				addiovec_error(NULL, "432 s_pipe() failed.");
+				addiovec_error(NULL, "432 libmail_streampipe() failed.");
 				exit(1);
 			}
 
 			couriertls_init(&cinfo);
 
 			strcat(strcpy(buf1, "-localfd="),
-			       str_size_t(pipefd[1], buf2));
+			       libmail_str_size_t(pipefd[1], buf2));
 
 			argvec[0]=buf1;
 			argvec[1]="-tcpd";
@@ -1012,7 +1112,7 @@ int	authenticated=0;
 			strncat(tlsbuf, cinfo.version, 40);
 			if (cinfo.bits > 0)
 				strcat(strcat(strcat(tlsbuf, ","),
-					      str_size_t(cinfo.bits,
+					      libmail_str_size_t(cinfo.bits,
 							 buf2)),
 				       "bits");
 			strncat(strcat(tlsbuf, ","), cinfo.cipher, 40);
@@ -1058,7 +1158,11 @@ int	authenticated=0;
 			authmethod=strtok(0, " \t\r\n");
 			initreply=strtok(0, " \t\r\n");
 
-			q=getenv("ESMTPAUTH");
+			if (tls && (q=getenv("ESMTPAUTH_TLS")) != 0 && *q)
+				;
+			else
+				q=getenv("ESMTPAUTH");
+
 			if (q == 0 || *q == 0)	authmethod=0;
 
 			if (authmethod == 0 || *authmethod == 0)
@@ -1147,6 +1251,7 @@ int	authenticated=0;
 					strlen(rc));
 				strcat(strcpy(p, "RELAYCLIENT="), rc);
 				putenv(p);
+				putenv("FAXRELAYCLIENT=");
 				startsubmit(tls);
 				authenticated=1;
 			}
