@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -7,6 +7,7 @@
 #include	"config.h"
 #endif
 #include	"soxwrap/soxwrap.h"
+#include	"soxwrap/sconnect.h"
 #include	"courier.h"
 #include	"smtproutes.h"
 #include	"localstatedir.h"
@@ -118,6 +119,7 @@ static void sock_timeout(unsigned);
 				&& p[3] == ' ')
 
 extern struct rw_list *esmtp_rw_install(const struct rw_install_info *);
+extern int isloopback(const char *);
 
 static void (*rewrite_func)(struct rw_info *, void (*)(struct rw_info *));
 
@@ -196,7 +198,7 @@ unsigned long mypid=(unsigned long)getpid();
 		{
 		char	pidbuf[NUMBUFSIZE];
 
-			printf("%u %s\n", childnum, str_pid_t(mypid, pidbuf));
+			printf("%u %s\n", childnum, libmail_str_pid_t(mypid, pidbuf));
 			fflush(stdout);
 		}
 
@@ -266,6 +268,9 @@ static const char *want_security(struct ctlfile *ctf)
 {
 	const char *sec;
 
+	if (smtproutes_flags & ROUTE_NOSECURITY)
+		return (0);
+
 	if (smtproutes_flags & ROUTE_STARTTLS)
 		return ("STARTTLS");
 
@@ -278,80 +283,26 @@ static const char *want_security(struct ctlfile *ctf)
 	return (0);
 }
 
-/*
-**	Attempt to bring up a socket to the remote server at a specific
-**	IP address.
-*/
-
-static int doconnect(int sockfd, const struct sockaddr *addr, size_t addr_s)
+static int get_sourceaddr(int af, RFC1035_NETADDR *addrbuf,
+	const struct sockaddr **addrptr, int *addrptrlen)
 {
-fd_set	fdr;
-struct	timeval	tv;
-int	rc;
+	int rc;
+	RFC1035_ADDR in;
+	const char *buf=getenv("SOURCE_ADDRESS");
 
-#ifdef  SOL_KEEPALIVE
-	setsockopt(sockfd, SOL_SOCKET, SOL_KEEPALIVE,
-		(const char *)&dummy, sizeof(dummy));
-#endif
+	if (buf && strcmp(buf, "0")) {
+		rc = rfc1035_aton(buf, &in);
+		if (rc != 0)
+			return rc;
+	} else
+		in = RFC1035_ADDRANY;
 
-#ifdef  SOL_LINGER
-	{
-	struct linger l;
+	rc = rfc1035_mkaddress(af, addrbuf, &in, htons(0),
+			addrptr, addrptrlen);
+	if (rc != 0)
+		return rc;
 
-		l.l_onoff=0;
-		l.l_linger=0;
-
-		setsockopt(sockfd, SOL_SOCKET, SOL_LINGER,
-			(const char *)&l, sizeof(l));
-	}
-#endif
-
-	/*
-	** If configuration says to use the kernel's timeout settings,
-	** just call connect, and be done with it.
-	*/
-
-	if (connect_timeout == 0)
-		return ( sox_connect(sockfd, addr, addr_s));
-
-	/* Asynchronous connect with timeout. */
-
-	if (fcntl(sockfd, F_SETFL, O_NONBLOCK) < 0)	return (-1);
-
-	if ( sox_connect(sockfd, addr, addr_s) == 0)
-	{
-		/* That was easy, we're done. */
-
-		if (fcntl(sockfd, F_SETFL, 0) < 0)	return (-1);
-		return (0);
-	}
-
-	if ( errno != EINPROGRESS && errno != EWOULDBLOCK)
-		return (-1);
-
-	/* Wait for the connection to go through, until the timeout expires */
-
-	FD_ZERO(&fdr);
-	FD_SET(sockfd, &fdr);
-	tv.tv_sec=connect_timeout;
-	tv.tv_usec=0;
-
-	rc=sox_select(sockfd+1, 0, &fdr, 0, &tv);
-	if (rc < 0)	return (-1);
-
-	if (!FD_ISSET(sockfd, &fdr))
-	{
-		errno=ETIMEDOUT;
-		return (-1);
-	}
-
-	if ( sox_connect(sockfd, addr, addr_s))
-	{
-		if (errno != EISCONN)	return (-1);
-	}
-
-	if (fcntl(sockfd, F_SETFL, 0) < 0)	return (-1);
-	return (0);
+	return 0;
 }
 
 /* Attempt to deliver a message */
@@ -416,6 +367,7 @@ char *smtproute;
 	if (sockfd < 0)	/* First time, connect to a server */
 	{
 	struct rfc1035_mxlist *mxlist, *p, *q;
+	int static_route= smtproute != NULL;
 
 		errno=0;	/* Detect network failures */
 
@@ -469,7 +421,9 @@ char *smtproute;
 				return;
 			}
 		
-			if (!q && config_islocal(p->hostname, 0))
+			if (!q && !static_route &&
+			    (config_islocal(p->hostname, 0)
+			     || isloopback(buf)))
 				q=p;
 		}
 
@@ -487,9 +441,9 @@ char *smtproute;
 		RFC1035_ADDR addr;
 		int	port;
 		int	af;
-		RFC1035_NETADDR addrbuf;
-		const struct sockaddr *saddr;
-		int	saddrlen;
+		RFC1035_NETADDR addrbuf, saddrbuf;
+		const struct sockaddr *addrptr, *saddrptr;
+		int	addrptrlen, saddrptrlen;
 
 			if (q && q->priority == p->priority)
 				break;
@@ -514,8 +468,13 @@ char *smtproute;
 			if ((sockfd=rfc1035_mksocket(SOCK_STREAM, 0, &af))
 					>= 0 &&
 				rfc1035_mkaddress(af, &addrbuf, &addr, port,
-					&saddr, &saddrlen) == 0 &&
-				doconnect(sockfd, saddr, saddrlen) == 0)
+					&addrptr, &addrptrlen) == 0 &&
+				get_sourceaddr(af, &saddrbuf, &saddrptr,
+					&saddrptrlen) == 0 &&
+				rfc1035_bindsource(sockfd, saddrptr,
+					saddrptrlen) == 0 &&
+				s_connect(sockfd, addrptr, addrptrlen,
+					  connect_timeout) == 0)
 			{
 				/*
 				** If we're connected, make sure EHLO or HELO
@@ -1231,7 +1190,7 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 				has8bitmime=1;
 			if (strcmp(hellobuf, "SIZE") == 0)
 				hassize=1;
-			if (strcmp(hellobuf, "STARTTLS") == 0)
+			if (strcmp(hellobuf, "STARTTLS") == 0 && !(smtproutes_flags & ROUTE_NOSECURITY))
 				hasstarttls=1;
 
 			if (strcmp(hellobuf, "AUTH") == 0
@@ -1268,7 +1227,17 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 				hasexdata=1;
 
 			if (IS_VERP_KEYWORD)
-				hasverp=1;
+			{
+				char *p=strchr(hellobuf, '=');
+
+				if (p)
+				{
+					for (++p; (p=strtok(p, ",")) != 0; p=0)
+						if (strcasecmp(p, "Courier")
+						    == 0)
+							hasverp=1;
+				}
+			}
 
 			if (IS_COURIER_EXTENSIONS)
 				hascourier=1;
@@ -1280,7 +1249,7 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 				if (p)
 				{
 					for (++p; (p=strtok(p, ",")) != 0; p=0)
-						if (strcmp(p, "STARTTLS") == 0)
+						if (strcmp(p, "STARTTLS") == 0 && !(smtproutes_flags & ROUTE_NOSECURITY))
 							hassecurity_starttls=1;
 				}
 			}
@@ -1336,7 +1305,6 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	const char *sec;
 	struct couriertls_info cinfo;
 	char	*verify_domain=0;
-	char	*verifyfailmsg=0;
 	char	localfd_buf[NUMBUFSIZE+30];
 	char	remotefd_buf[NUMBUFSIZE+30];
 	char	miscbuf[NUMBUFSIZE];
@@ -1349,13 +1317,9 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	int restore_origcert=0;
 	int n;
 
-	static const char verifyfailmsgtxt[]=
-		"-verifyfailmsg=Unable to verify SSL certificate"
-		" for %s\n";
-
-	if (s_pipe(pipefd))
+	if (libmail_streampipe(pipefd))
 	{
-		perror("s_pipe");
+		perror("libmail_streampipe");
 		return (-1);
 	}
 
@@ -1398,9 +1362,9 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	fcntl(pipefd[0], F_SETFD, FD_CLOEXEC);
 
 	strcat(strcpy(localfd_buf, "-localfd="),
-	       str_size_t(pipefd[1], miscbuf));
+	       libmail_str_size_t(pipefd[1], miscbuf));
 	strcat(strcpy(remotefd_buf, "-remotefd="),
-	       str_size_t(sockfd, miscbuf));
+	       libmail_str_size_t(sockfd, miscbuf));
 
 	p=getenv("ESMTP_TLS_VERIFY_DOMAIN");
 
@@ -1457,10 +1421,6 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		verify_domain=courier_malloc(sizeof("-verify=")
 					     +strlen(hostname));
 		strcat(strcpy(verify_domain, "-verify="), hostname);
-
-		verifyfailmsg=courier_malloc(sizeof(verifyfailmsgtxt)+
-					     strlen(hostname));
-		sprintf(verifyfailmsg, verifyfailmsgtxt, hostname);
 	}
 
 
@@ -1471,7 +1431,6 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 	if (verify_domain)
 	{
 		argvec[n++]=verify_domain;
-		argvec[n++]=verifyfailmsg;
 	}
 	argvec[n]=0;
 
@@ -1481,13 +1440,18 @@ static int starttls(struct moduledel *del, struct ctlfile *ctf,
 		putenv(origcert_buf);
 	if (verify_domain)
 		free(verify_domain);
-	if (verifyfailmsg)
-		free(verifyfailmsg);
 
 	close(sockfd);
 	sockfd=pipefd[0];
 	close(pipefd[1]);
 
+	if (!n && fcntl(sockfd, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		n= -1;
+		strcpy(cinfo.errmsg, "fcntl() failed");
+	}
+
 	if (n)
 	{
 		char tmperrbuf[sizeof(cinfo.errmsg)+10];
@@ -2434,59 +2398,6 @@ int	n;
 		rfc2045_parse(rfcp, buf, n);
 }
 
-/* The server we're talking to is broken.  Make a note of that fact, in BOFH
-** mode.
-*/
-
-static void bofhbadserver()
-{
-const char *h=courierdir();
-char	namebuf[RFC1035_NTOABUFSIZE];
-char	*p;
-struct	stat	stat_buf;
-
-	sockipname(namebuf);
-
-	p=courier_malloc(sizeof(TMPDIR "/broken/")+strlen(h)+
-		strlen(namebuf));
-	strcat(strcat(strcpy(p, h), TMPDIR "/broken/"), namebuf);
-
-#if	HAVE_UTIME
-	if (stat(p, &stat_buf) == 0 && stat_buf.st_size >= 256)
-	{
-	struct utimbuf utb;
-
-		time(&utb.actime);
-		utb.modtime=utb.actime;
-		utime(p, &utb);
-	}
-	else
-#else
-#if	HAVE_UTIMES
-	if (stat(p, &stat_buf) == 0 && stat_buf.st_size >= 256)
-	{
-	struct timeval tv;
-
-		time(&tv.tv_sec);
-		tv.tv_usec=0;
-		utimes(p, &tv);
-	}
-	else
-#endif
-#endif
-	{
-	int	f=open(p, O_WRONLY|O_APPEND|O_CREAT, 0644);
-
-		if (f >= 0)
-		{
-			sox_write(f, "", 1);
-			sox_close(f);
-		}
-	}
-
-	free(p);
-}
-
 /*
 ** Ok, everything above is collected into a nice, tight, package.
 */
@@ -2552,34 +2463,9 @@ int	is8bitmsg;
 
 	if (smtpreply(mailfroms, del, ctf, 1))	/* MAIL FROM rejected */
 	{
-	const char *q;
-
 		if (rfcp)	rfc2045_free(rfcp);
 		sox_close(fd);
 
-		/* Check if this is a misconfigured server we're talking to. */
-
-		if (strncmp(mailfroms, "MAIL FROM:<>", 12) == 0 &&
-			(q=getenv("BOFHBANBROKENSERVERS")) &&
-			*q != '0')
-		{
-		char	*p=strcat(strcat(strcpy(
-				courier_malloc(sizeof("MAIL FROM:<nobody@>\r\n")
-					+strlen(host)),
-				"MAIL FROM:<nobody@"), host), ">\r\n");
-
-			if (dowritestr(p) == 0 && writeflush() == 0)
-			{
-				if ((q=readline()) != 0 &&
-					SMTPREPLY_TYPE(q)
-						== COMCTLFILE_DELSUCCESS)
-					bofhbadserver();
-				while (q && !ISFINALLINE(q))
-					q=readline();
-			}
-			free(p);
-		}
-
 		free(mailfroms);
 		return;
 	}
