@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2004 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -25,12 +25,13 @@
 #include	"authlib/auth.h"
 #include	"authlib/authmod.h"
 #include	"tcpd/spipe.h"
+#include	"authlib/debug.h"
 
-static const char rcsid[]="$Id: imaplogin.c,v 1.10 2001/07/24 12:30:00 mrsam Exp $";
+static const char rcsid[]="$Id: imaplogin.c,v 1.23 2004/04/11 23:14:03 mrsam Exp $";
 
 FILE *debugfile=0;
 extern void mainloop();
-extern void capability();
+extern void imapcapability();
 extern int have_starttls();
 extern int tlsrequired();
 extern int authenticate(const char *);
@@ -45,24 +46,25 @@ void rfc2045_error(const char *p)
 	_exit(0);
 }
 
+extern void cmdfail(const char *, const char *);
+extern void cmdsuccess(const char *, const char *);
+
 static int	starttls(const char *tag)
 {
 int	pipefd[2];
 pid_t	p;
 int	waitstat;
 
-	if (s_pipe(pipefd))
+	if (libmail_streampipe(pipefd))
 	{
-		writes(tag);
-		writes(" NO s_pipe() failed.");
+		cmdfail(tag, "libmail_streampipe() failed.\r\n");
 		return (-1);
 	}
 
 	p=fork();
 	if (p == -1)
 	{
-		writes(tag);
-		writes(" NO fork() failed.");
+		cmdfail(tag, "fork() failed.\r\n");
 		return (-1);
 	}
 
@@ -99,8 +101,7 @@ int	waitstat;
 			buf1, "-tcpd", "-server", (char *)0);
 	}
 
-	writes(tag);
-	writes(" OK Begin SSL/TLS negotation now.\r\n");
+	cmdsuccess(tag, "Begin SSL/TLS negotiation now.\r\n");
 	writeflush();
 	close(pipefd[1]);
 	close(0);
@@ -114,19 +115,42 @@ int	waitstat;
 	write(1, "", 1);	/* child - exec OK now */
 	while (wait(&waitstat) != p)
 		;
+
+	/* We use select() with a timeout, so use non-blocking filedescs */
+
+	if (fcntl(0, F_SETFL, O_NONBLOCK) ||
+	    fcntl(1, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		exit(1);
+	}
 	return (0);
 }
 
 int do_imap_command(const char *tag)
 {
-struct	imaptoken *curtoken=nexttoken();
+	struct	imaptoken *curtoken=nexttoken();
+	char authservice[40];
+
+#if SMAP
+	if (strcmp(tag, "\\SMAP1") == 0)
+	{
+		const char *p=getenv("SMAP_CAPABILITY");
+
+		if (p && *p)
+			putenv("PROTOCOL=SMAP1");
+		else
+			return -1;
+	}
+#endif
+
+	auth_debug_login( 1, "command=%s", curtoken->tokenbuf );
 
 	if (strcmp(curtoken->tokenbuf, "LOGOUT") == 0)
 	{
 		if (nexttoken()->tokentype != IT_EOL)   return (-1);
 		writes("* BYE Courier-IMAP server shutting down\r\n");
-		writes(tag);
-		writes(" OK LOGOUT completed\r\n");
+		cmdsuccess(tag, "LOGOUT completed\r\n");
 		writeflush();
 		fprintf(stderr, "INFO: LOGOUT, ip=[%s]\n",
 			getenv("TCPREMOTEIP"));
@@ -135,16 +159,17 @@ struct	imaptoken *curtoken=nexttoken();
 	if (strcmp(curtoken->tokenbuf, "NOOP") == 0)
 	{
 		if (nexttoken()->tokentype != IT_EOL)	return (-1);
-		writes(tag);
-		writes(" OK NOOP completed\r\n");
+		cmdsuccess(tag, "NOOP completed\r\n");
 		return (0);
 	}
 	if (strcmp(curtoken->tokenbuf, "CAPABILITY") == 0)
 	{
 		if (nexttoken()->tokentype != IT_EOL)	return (-1);
-		capability();
-		writes(tag);
-		writes(" OK CAPABILITY completed\r\n");
+
+		writes("* CAPABILITY ");
+		imapcapability();
+		writes("\r\n");
+		cmdsuccess(tag, "CAPABILITY completed\r\n");
 		return (0);
 	}
 
@@ -164,11 +189,11 @@ struct	imaptoken *curtoken=nexttoken();
 	char	*userid;
 	char	*tagenv;
 	char	*passwd;
+	const char *p;
 
 		if (have_starttls() && tlsrequired())	/* Not yet */
 		{
-			writes(tag);
-			writes(" NO STARTTLS required\r\n");
+			cmdfail(tag, "STARTTLS required\r\n");
 			return (0);
 		}
 
@@ -184,7 +209,7 @@ struct	imaptoken *curtoken=nexttoken();
 		userid=strdup(tok->tokenbuf);
 		if (!userid)
 			write_error_exit(0);
-		tok=nexttoken_nouc();
+		tok=nexttoken_nouc_okbracket();
 		switch (tok->tokentype)	{
 		case IT_ATOM:
 		case IT_NUMBER:
@@ -208,21 +233,28 @@ struct	imaptoken *curtoken=nexttoken();
 			return (-1);
 		}
 		putenv(tagenv);
-		authmod_login(main_argc-1, main_argv+1, "imap",
-			userid, passwd);
+
+		strcat(strcpy(authservice, "AUTHSERVICE"),
+		       getenv("TCPLOCALPORT"));
+
+		p=getenv(authservice);
+
+		if (!p || !*p)
+			p="imap";
+
+		authmod_login(main_argc-1, main_argv+1, p,
+                         userid, passwd);
 	}
 
 	if (strcmp(curtoken->tokenbuf, "AUTHENTICATE") == 0)
 	{
 		if (have_starttls() && tlsrequired())	/* Not yet */
 		{
-			writes(tag);
-			writes(" NO STARTTLS required\r\n");
+			cmdfail(tag, "STARTTLS required\r\n");
 			return (0);
 		}
 		authenticate(tag);
-		writes(tag);
-		writes(" NO Authentication failed.\r\n");
+		cmdfail(tag, "Authentication failed.\r\n");
 		writeflush();
 		return (-2);
 	}
@@ -246,15 +278,29 @@ const char	*ip;
 
 #endif
 
+	auth_debug_login_init();
+
+	/* We use select() with a timeout, so use non-blocking filedescs */
+
+	if (fcntl(0, F_SETFL, O_NONBLOCK) ||
+	    fcntl(1, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		exit(1);
+	}
+
 	if (authmoduser(argc, argv, 60, 5))
 	{
-		writes("* OK Courier-IMAP ready. Copyright 1998-2001 Double Precision, Inc.  See COPYING for distribution information.\r\n");
-		fprintf(stderr, "INFO: Connection, ip=[%s]\n", ip);
+		writes("* OK [CAPABILITY ");
+		imapcapability();
+		writes("] Courier-IMAP ready. "
+		       "Copyright 1998-2004 Double Precision, Inc.  "
+		       "See COPYING for distribution information.\r\n");
+		fprintf(stderr, "DEBUG: Connection, ip=[%s]\n", ip);
 	}
 	else
 	{
-		writes(tag ? tag:"");
-		writes(" NO Login failed.\r\n");
+		cmdfail(tag ? tag:"", "Login failed.\r\n");
 		fprintf(stderr, "ERR: LOGIN FAILED, ip=[%s]\n", ip);
 	}
 
@@ -262,6 +308,13 @@ const char	*ip;
 	main_argc=argc;
 	main_argv=argv;
 
+	putenv("PROTOCOL=IMAP");
+
 	mainloop();
 	return (0);
 }
+
+void bye()
+{
+	exit(0);
+}
