@@ -11,8 +11,7 @@ FILE	*fp=fopen(f, "a");
 
 	if (!fp)	return (-1);
 
-	fprintf(fp, "%s\t%s\t%s\t%s\t%s\t%s\n", a->name, a->host, a->port,
-		a->suffix, a->binddn, a->bindpw);
+	ldapabook_writerec(a, fp);
 
 	if (fflush(fp) || fclose(fp))
 	{
