@@ -1,12 +1,13 @@
 /*
-** Copyright 2000 Double Precision, Inc.
+** Copyright 2000-2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
 #include	<sys/types.h>
 #include	<string.h>
+#include	"config.h"
 
-static const char rcsid[]="$Id: maildirflags.c,v 1.1 2000/10/07 01:10:19 mrsam Exp $";
+static const char rcsid[]="$Id: maildirflags.c,v 1.3 2002/10/01 03:16:25 mrsam Exp $";
 
 int maildir_hasflag(const char *filename, char flag)
 {
@@ -15,8 +16,8 @@ int maildir_hasflag(const char *filename, char flag)
 	if (p)
 		filename=p+1;
 
-	p=strrchr(p, ':');
-	if (p && strncmp(p, ":2,", 3) == 0 &&
+	p=strrchr(filename, MDIRSEP[0]);
+	if (p && strncmp(p, MDIRSEP "2,", 3) == 0 &&
 	    strchr(p+3, flag))
 		return (1);
 	return (0);
