@@ -14,11 +14,14 @@ extern "C" {
 #define MYSQL_DEFAULT_MAILDIR_FIELD "maildir"
 #define MYSQL_DEFAULT_HOMEDIRECTORY_FIELD "homedir"
 #define MYSQL_DEFAULT_QUOTA_FIELD "quota"
-#define MYSQL_DEFAULT_STATUS_FIELD "vmailstatus"
+#define MYSQL_DEFAULT_WHERE_CLAUSE ""
 #define MYSQL_DEFAULT_TIMEOUT 5
 
+#define MYSQL_HOSTNAMES 5
+
 typedef struct {
-  char *hostname;
+  /* allow some hostname options */
+  char *hostname[MYSQL_HOSTNAMES];
   unsigned short port;
   char *socket;
   char *database;
@@ -32,10 +35,11 @@ typedef struct {
   char *homedir_field;
   char *maildir_field;
   char *quota_field;
-  char *mailstatus_field;
+  char *where_clause;
 
   int default_uidnumber;
   int default_gidnumber;
+  char *default_quota;
 
   int timeout;
 } mdmysqlconfig;
