@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2003 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -50,15 +50,15 @@
 #include	"liblock/config.h"
 #include	"liblock/liblock.h"
 #include	"tcpremoteinfo.h"
-#include	"authlib/auth.h"
 #include	"numlib/numlib.h"
 #include	"argparse.h"
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.26 2001/08/16 01:38:43 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.37 2004/01/24 20:10:57 mrsam Exp $";
 
 static const char *accessarg=0;
+static const char *denymsgarg=0;
 static const char *listenarg=0;
 static const char *ipaddrarg=0;
 static const char *userarg=0;
@@ -83,6 +83,7 @@ static void setup_block(const char *);
 
 static struct args arginfo[]={
 	{"access", &accessarg},
+	{"denymsg", &denymsgarg},
 	{"address", &ipaddrarg},
 	{"block", 0, setup_block},
 	{"group", &grouparg},
@@ -193,6 +194,8 @@ static RETSIGTYPE sighup(int n)
 {
 	sighup_received=1;
 
+	signal(SIGHUP, sighup);
+
 #if RETSIGTYPE != void
 	return (0)
 #endif
@@ -225,7 +228,7 @@ static int parseaddr(const char *p)
 	char *buf=strdup(p);
 	char *q, *a, *s;
 
-	if (!p)
+	if (!buf)
 	{
 		perror("malloc");
 		return (-1);
@@ -401,7 +404,7 @@ static int mksocket(const char *ipaddrarg,	/* Host/IP address */
 
 	if (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
 	{
-		if (flags && MKS_ERROK)
+		if (flags & MKS_ERROK)
 		{
 			close(fd);
 			return (-2);
@@ -440,7 +443,21 @@ static int mksockets()
 
 	for (p=fdlist; p; p=p->next)
 	{
-		int fd=mksocket(p->ipaddr, p->servname, 0);
+		int fd;
+		struct in_addr addr;
+		int fd_flag=0;
+
+		if (p->ipaddr && strcmp(p->ipaddr, "0"))
+		{
+			/* FreeBSD needs AF_INET binds for IPv4 addys */
+
+			if (rfc1035_aton_ipv4(p->ipaddr, &addr) == 0)
+			{
+				fd_flag=MKS_USEAFINET4;
+			}
+		}
+
+		fd=mksocket(p->ipaddr, p->servname, fd_flag);
 
 		if (fd < 0)
 			break;
@@ -607,7 +624,7 @@ int	lockfd;
 		}
 		else	gid=gr->gr_gid;
 
-		authchangegroup(gid);
+		libmail_changegroup(gid);
 	}
 
 	if (userarg)
@@ -617,14 +634,14 @@ int	lockfd;
 		if (isid(userarg))
 		{
 			uid=atoi(userarg);
-			authchangeuidgid(uid, getgid());
+			libmail_changeuidgid(uid, getgid());
 		}
 		else
 		{
 		gid_t	g=getgid(), *gp=0;
 
 			if (grouparg)	gp= &g;
-			authchangeusername(userarg, gp);
+			libmail_changeusername(userarg, gp);
 		}
 	}
 
@@ -865,6 +882,7 @@ int	l;
 		}
 		*accessptr=0;
 	}
+
 	if (strncmp(accessptr, "deny", 4) == 0)
 	{
 		free(accessptr);
@@ -917,7 +935,8 @@ int	l;
 		}
 
 		while (*q && isspace((int)(unsigned char)*q))	++q;
-		while (l=strlen(q)-1, isspace((int)(unsigned char)q[l-1]))
+		while ((l=strlen(q)) > 0
+		       && isspace((int)(unsigned char)q[l-1]))
 			q[--l]=0;
 		putenv(q);
 	}
@@ -1143,6 +1162,10 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 
 		if (allowaccess(&addr) == 0)
 		{
+			if (denymsgarg) {
+				write(sockfd, denymsgarg, strlen(denymsgarg));
+				write(sockfd, "\n", 1);
+			}
 			sox_close(sockfd);
 			_exit(0);
 		}
@@ -1359,7 +1382,7 @@ int i;
 
 	if (!varname)	varname="BLOCK";
 
-	if ((q=getenv(varname)) != 0 && *q)	return;
+	if ((q=getenv(varname)) != 0)	return;
 					/* Env var already set */
 
 	if (p->ia.s_addr == INADDR_ANY)
@@ -1475,9 +1498,10 @@ int i;
 
 			/* No predefined message, look for a TXT record. */
 
-			if ((j=rfc1035_replysearch_all(replyp, hostname,
-				RFC1035_TYPE_TXT,
-				RFC1035_CLASS_IN, 0)) >= 0)
+			if ((j=rfc1035_replysearch_all(&rfc1035_default_resolver,
+						       replyp, hostname,
+						       RFC1035_TYPE_TXT,
+						       RFC1035_CLASS_IN, 0)) >= 0)
 			{
 				rfc1035_rr_gettxt(replyp->allrrs[j], 0, buf);
 				mysetenv(vvarname, buf);
