@@ -1,6 +1,6 @@
 #! perl
 #
-# $Id: admin-main.pl,v 1.1 2001/06/23 22:34:08 mrsam Exp $
+# $Id: admin-main.pl,v 1.2 2003/07/06 16:16:37 mrsam Exp $
 #
 # Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
@@ -41,7 +41,7 @@ if (opendir($fh, $webadmindir))
     close($fh);
 }
 
-my $menuhtml="<table border=0 cellspacing=8>";
+my $menuhtml="<table border=\"0\" cellspacing=\"8\">";
 
 foreach (sort keys %mainmenu)
 {
