@@ -1,11 +1,11 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: gpg.c,v 1.8 2001/10/01 22:52:25 mrsam Exp $
+** $Id: gpg.c,v 1.12 2003/06/17 15:38:37 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"config.h"
@@ -59,6 +59,11 @@ static int gpg_error(const char *p, size_t l, void *dummy)
 	return (0);
 }
 
+static void gpg_error_save(const char *errmsg, void *dummy)
+{
+	gpg_error(errmsg, strlen(errmsg), dummy);
+}
+
 int gpgbadarg(const char *p)
 {
 	for ( ; *p; p++)
@@ -75,9 +80,9 @@ static void dump_error()
 {
 	if (gpgerrcnt >= 0)
 	{
-		printf("<FONT COLOR=\"#E00000\"><PRE CLASS=\"gpgerroutput\">");
+		printf("<span style=\"color: #e00000\"><pre class=\"gpgerroutput\">");
 		print_attrencodedlen (gpgerrbuf, gpgerrcnt, 1, stdout);
-		printf("</PRE></FONT>\n");
+		printf("</pre></span>\n");
 	}
 }
 
@@ -92,17 +97,17 @@ static int show_key(const char *fingerprint, const char *shortname,
 {
 	struct listinfo *li=(struct listinfo *)gli->voidarg;
 
-	printf("<TR VALIGN=MIDDLE CLASS=\"%s\"><TD>"
-	       "<input type=radio name=%s value=\"",
+	printf("<tr valign=\"middle\" class=\"%s\"><td>"
+	       "<input type=\"radio\" name=\"%s\" value=\"",
 	       li->issecret ? "gpgseckey":"gpgpubkey",
 	       li->issecret ? "seckeyname":"pubkeyname");
 
 	print_attrencodedlen(fingerprint, strlen(fingerprint), 0, stdout);
-	printf("\"%s></TD><TD><TT>",
+	printf("\"%s /></td><td><span class=\"tt\">",
 	       li->default_key && strcmp(li->default_key, fingerprint) == 0
-	       ? " CHECKED":"");
+	       ? " checked=\"checked\"":"");
 	print_safe(key);
-	printf("</TT></TD></TR>\n");
+	printf("</span></td></tr>\n");
 	return (0);
 }
 
@@ -137,7 +142,7 @@ static void listpubsec(int flag,
 
 	gpginiterr();
 
-	rc=gpg_listkeys(GPGDIR, flag, callback_func, gpg_error, &gli);
+	rc=libmail_gpg_listkeys(GPGDIR, flag, callback_func, gpg_error, &gli);
 
 	if (rc)
 	{
@@ -147,14 +152,14 @@ static void listpubsec(int flag,
 
 void gpglistpub()
 {
-	printf("<table width=\"100%%\" border=0 cellspacing=2 cellpadding=0 class=\"gpgpubkeys\">");
+	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"2\" cellpadding=\"0\" class=\"gpgpubkeys\">");
 	listpubsec(0, show_key, NULL);
 	printf("</table>");
 }
 
 void gpglistsec()
 {
-	printf("<table width=\"100%%\" border=0 cellspacing=2 cellpadding=0 class=\"gpgseckeys\">");
+	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"2\" cellpadding=\"0\" class=\"gpgseckeys\">");
 	listpubsec(1, show_key, NULL);
 	printf("</table>");
 }
@@ -164,11 +169,12 @@ static int select_key(const char *fingerprint, const char *shortname,
 		      struct gpg_list_info *gli,
 		      int is_select)
 {
-	printf("<OPTION VALUE=\"");
+	printf("<option value=\"");
 	print_attrencodedlen(fingerprint, strlen(fingerprint), 0, stdout);
-	printf("\"%s>", is_select ? " SELECTED":"");
+	printf("\"%s>", is_select ? " selected='selected'":"");
 
 	print_safe(shortname);
+	printf("</option>");
 	return (0);
 }
 
@@ -266,9 +272,9 @@ static int knownkey(const char *shortname, const char *known_keys)
 }
 
 static int encrypt_key_default(const char *fingerprint, const char *shortname,
-			      const char *key,
-			      int invalid,
-			      struct gpg_list_info *gli)
+			       const char *key,
+			       int invalid,
+			       struct gpg_list_info *gli)
 {
 	struct listinfo *li=(struct listinfo *)gli->voidarg;
 
@@ -348,7 +354,8 @@ void gpgcreate()
 		u= &unicode_ISO8859_1;
 
 	if (*newname == 0 || *newaddress == 0 || strchr(newaddress, '@') == 0
-	    || gpgbadarg(newname) || gpgbadarg(newaddress) || gpgbadarg(newcomment)
+	    || gpgbadarg(newname) || gpgbadarg(newaddress)
+	    || gpgbadarg(newcomment)
 	    || ekl < 512 || ekl > 2048 || skl < 512 || skl > 1024)
 	{
 		printf("%s\n", getarg("BADARGS"));
@@ -370,17 +377,18 @@ void gpgcreate()
 		}
 	}
 
-	printf("<PRE CLASS=\"gpgcreate\">");
+	printf("<pre class=\"gpgcreate\">");
 
 	linelen=0;
 
-	gpg_genkey(GPGDIR, u, newname, newaddress, newcomment, skl, ekl,
-		   newexpire, newexpirewhen,
-		   passphrase,
-		   &dump_func,
-		   &timeout_func,
-		   &linelen);
-	printf("</PRE>");
+	libmail_gpg_genkey(GPGDIR, u, newname, newaddress, newcomment,
+			   skl, ekl,
+			   newexpire, newexpirewhen,
+			   passphrase,
+			   &dump_func,
+			   &timeout_func,
+			   &linelen);
+	printf("</pre>");
 }
 
 static void delkey(const char *keyname, int flag)
@@ -392,13 +400,13 @@ static void delkey(const char *keyname, int flag)
 
 	gpginiterr();
 
-	rc=gpg_deletekey(GPGDIR, flag, keyname, gpg_error, NULL);
+	rc=libmail_gpg_deletekey(GPGDIR, flag, keyname, gpg_error, NULL);
 
 	if (rc)
 	{
-		printf("<blockquote>%s\n", getarg("DELETEFAIL"));
+		printf("<div class=\"indent\">%s\n", getarg("DELETEFAIL"));
 		dump_error();
-		printf("</blockquote>\n");
+		printf("</div>\n");
 	}
 }
 
@@ -426,11 +434,17 @@ static FILE *passphrasefp()
 	return (fp);
 }
 
-static void signkey(const char *signthis, const char *signwith)
+static void signkey(const char *signthis, const char *signwith,
+		    const char *trustlevel)
 {
 	int rc;
 	FILE *fp=NULL;
 
+	int n=atoi(trustlevel);
+
+	if (n < 0 || n > 9)
+		n=0;
+
 	if (gpgbadarg(signthis) || gpgbadarg(signwith))
 		return;
 
@@ -439,17 +453,17 @@ static void signkey(const char *signthis, const char *signwith)
 
 	fp=passphrasefp();
 
-	rc=gpg_signkey(GPGDIR, signthis, signwith,
-		       fp ? fileno(fp):-1, gpg_error, NULL);
+	rc=libmail_gpg_signkey(GPGDIR, signthis, signwith,
+			       fp ? fileno(fp):-1, gpg_error, n, NULL);
 
 	if (fp)
 		fclose(fp);
 
 	if (rc)
 	{
-		printf("<blockquote>%s\n", getarg("SIGNFAIL"));
+		printf("<div class=\"indent\">%s\n", getarg("SIGNFAIL"));
 		dump_error();
-		printf("</blockquote>\n");
+		printf("</div>\n");
 	}
 }
 
@@ -468,38 +482,48 @@ void gpgdo()
 	else if (*cgi("delsec") && *cgi("really"))
 		delkey(cgi("seckeyname"), 1);
 	else if (*cgi("sign"))
-		signkey(cgi("pubkeyname"), cgi("seckeyname"));
+		signkey(cgi("pubkeyname"), cgi("seckeyname"),
+			cgi("signlevel"));
 	else if (*cgi("setdefault"))
 		setdefault(cgi("seckeyname"));
 }
 
 static char gpgerrbuf[1024];
 
-static int wait_mimegpg(pid_t pid, int errfd)
+static int read_fd(char *buf, size_t cnt, void *vp)
 {
-	pid_t pid2;
-	int waitstat;
-	char *p;
-	int i, n;
+	FILE *fp=(FILE *)vp;
+	size_t i;
+	int c;
 
-	p=gpgerrbuf;
-	n=sizeof(gpgerrbuf)-1;
+	if (cnt == 0)
+		return -1;
 
-	while (n && (i=read(errfd, p, n)) > 0)
+	--cnt;
+
+	for (i=0; i<cnt; i++)
 	{
-		p += i;
-		n -= i;
-	}
-	close(errfd);
-	*p=0;
+		if ((c=getc(fp)) == EOF)
+		{
+			if (i == 0)
+				return -1;
+			break;
+		}
+		buf[i]=c;
 
-	while ((pid2=wait(&waitstat)) != pid)
-	{
-		if (pid2 < 0 && errno == ECHILD)
-			return (-1);
+		if (c == '\n')
+		{
+			++i;
+			break;
+		}
 	}
+	buf[i]=0;
+	return 0;
+}
 
-	return (WIFEXITED(waitstat) && WEXITSTATUS(waitstat) == 0 ? 0:-1);
+static void write_fd(const char *p, size_t n, void *dummy)
+{
+	fwrite(p, n, 1, (FILE *)dummy);
 }
 
 int gpgdomsg(int in_fd, int out_fd, const char *signkey,
@@ -510,37 +534,43 @@ int gpgdomsg(int in_fd, int out_fd, const char *signkey,
 	int i;
 	char *p;
 	char **argvec;
-	pid_t pid;
-	int stderrfd[2];
 	FILE *passfd=NULL;
-	const char *cp;
 	char passfd_buf[NUMBUFSIZE];
+	struct libmail_gpg_info gi;
+
+	int in_dup, out_dup;
+	FILE *in_fp, *out_fp;
+
+	gpginiterr();
 
 	if (!k)
-		enomem();
-
-	if (pipe(stderrfd))
 	{
+		enomem();
+		return 1;
+	}
+
+	if ((in_dup=dup(in_fd)) < 0 || (in_fp=fdopen(in_dup, "r")) == NULL)
+	{
+		if (in_dup >= 0)
+			close(in_dup);
 		free(k);
 		enomem();
+		return 1;
 	}
 
-	if (*(cp=cgi("passphrase")) != 0)
+	if ((out_dup=dup(out_fd)) < 0 || (out_fp=fdopen(out_dup, "w")) == NULL)
 	{
-		passfd=tmpfile();
-		if (passfd)
-		{
-			fprintf(passfd, "%s", cp);
-			if (fflush(passfd) || ferror(passfd)
-			    || lseek(fileno(passfd), 0L, SEEK_SET) < 0
-			    || fcntl(fileno(passfd), F_SETFD, 0) < 0)
-			{
-				fclose(passfd);
-				passfd=NULL;
-			}
-		}
+		if (out_dup >= 0)
+			close(out_dup);
+		fclose(in_fp);
+		close(in_dup);
+		free(k);
+		enomem();
+		return 1;
 	}
 
+	passfd=passphrasefp();
+
 	n=0;
 	for (p=k; (p=strtok(p, " ")) != NULL; p=NULL)
 		++n;
@@ -548,25 +578,33 @@ int gpgdomsg(int in_fd, int out_fd, const char *signkey,
 	argvec=malloc((n * 2 + 22)*sizeof(char *));
 	if (!argvec)
 	{
-		close(stderrfd[0]);
-		close(stderrfd[1]);
+		fclose(out_fp);
+		close(out_dup);
+		fclose(in_fp);
+		close(in_dup);
 		free(k);
 		enomem();
+		return 1;
 	}
 
-	i=0;
-	argvec[i++]=MIMEGPG;
-	if (signkey)
-		argvec[i++]="-s";
-	if (n > 0)
-		argvec[i++]="-E";
+	memset(&gi, 0, sizeof(gi));
+
+	gi.gnupghome=GPGDIR;
 	if (passfd)
 	{
-		argvec[i++]="-p";
-		argvec[i++]=str_size_t(fileno(passfd), passfd_buf);
+		gi.passphrase_fd=libmail_str_size_t(fileno(passfd),
+						    passfd_buf);
 	}
 
-	argvec[i++] = "--";
+	gi.input_func= read_fd;
+	gi.input_func_arg= in_fp;
+	gi.output_func= write_fd;
+	gi.output_func_arg= out_fp;
+	gi.errhandler_func= gpg_error_save;
+	gi.errhandler_arg= NULL;
+
+
+	i=0;
 	argvec[i++] = "--no-tty";
 	if (signkey)
 	{
@@ -583,42 +621,23 @@ int gpgdomsg(int in_fd, int out_fd, const char *signkey,
 		argvec[i++]=p;
 	}
 	argvec[i]=0;
+	gi.argc=i;
+	gi.argv=argvec;
 
-	pid=fork();
-	if (pid < 0)
-	{
-		free(k);
-		close(stderrfd[0]);
-		close(stderrfd[1]);
-		return (-1);
-	}
+	i=libmail_gpg_signencode(signkey ? 1:0,
+				 n > 0 ? LIBMAIL_GPG_ENCAPSULATE:0,
+				 &gi);
 
-	if (pid == 0)
-	{
-		close(0);
-		dup(in_fd);
-		close(1);
-		dup(out_fd);
-		close(in_fd);
-		close(out_fd);
-
-		close(2);
-		dup(stderrfd[1]);
-		close(stderrfd[1]);
-		close(stderrfd[0]);
-
-		putenv("GNUPGHOME=" GPGDIR);
-		execv(argvec[0], argvec);
-		perror("mimegpg");
-		exit(1);
-	}
 	free(argvec);
+	fclose(out_fp);
+	close(out_dup);
+	fclose(in_fp);
+	close(in_dup);
 	free(k);
-	close(stderrfd[1]);
 	if (passfd)
 		fclose(passfd);
 
-	return ( wait_mimegpg(pid, stderrfd[0]));
+	return i;
 }
 
 void sent_gpgerrtxt()
@@ -650,73 +669,72 @@ void sent_gpgerrresume()
 
 int gpgdecode(int in_fd, int out_fd)
 {
-	pid_t pid;
-	int stderrfd[2];
-	int rc;
+	char passfd_buf[NUMBUFSIZE];
+	FILE *fp=passphrasefp();
+	int in_dup, out_dup;
+	FILE *in_fp, *out_fp;
+	struct libmail_gpg_info gi;
+	char *argvec[2];
+	int i;
 
-	if (pipe(stderrfd))
+	gpginiterr();
+
+	if ((in_dup=dup(in_fd)) < 0 || (in_fp=fdopen(in_dup, "r")) == NULL)
 	{
+		if (in_dup >= 0)
+			close(in_dup);
+		fclose(fp);
 		enomem();
+		return 1;
 	}
 
-	pid=fork();
-	if (pid < 0)
+	if ((out_dup=dup(out_fd)) < 0 || (out_fp=fdopen(out_dup, "w")) == NULL)
 	{
-		close(stderrfd[0]);
-		close(stderrfd[1]);
+		if (out_dup >= 0)
+			close(out_dup);
+		fclose(in_fp);
+		close(in_dup);
+		fclose(fp);
 		enomem();
+		return 1;
 	}
 
-	if (pid == 0)
+	memset(&gi, 0, sizeof(gi));
+
+	gi.gnupghome=GPGDIR;
+	if (fp)
 	{
-		FILE *fp=passphrasefp();
-		char *argvec[20];
-		int i;
-		char passfd_buf[NUMBUFSIZE];
-
-		close(0);
-		dup(in_fd);
-		close(1);
-		dup(out_fd);
-		close(in_fd);
-		close(out_fd);
-
-		close(2);
-		dup(stderrfd[1]);
-		close(stderrfd[1]);
-		close(stderrfd[0]);
-
-		argvec[0]=MIMEGPG;
-		argvec[1]="-c";
-		argvec[2]="-d";
-
-		i=3;
-
-		if (fp)
-		{
-			argvec[i++]="-p";
-			argvec[i++]=str_size_t(fileno(fp), passfd_buf);
-		}
-
-		argvec[i++]="--";
-		argvec[i++]="--no-tty";
-		argvec[i]=0;
-
-		putenv("GNUPGHOME=" GPGDIR);
-		execv(MIMEGPG, argvec);
-		perror(MIMEGPG);
-		exit(1);
+		gi.passphrase_fd=libmail_str_size_t(fileno(fp), passfd_buf);
 	}
-	close (stderrfd[1]);
-	rc=wait_mimegpg(pid, stderrfd[0]);
 
-	if (rc)
+	gi.input_func= read_fd;
+	gi.input_func_arg= in_fp;
+	gi.output_func= write_fd;
+	gi.output_func_arg= out_fp;
+	gi.errhandler_func= gpg_error_save;
+	gi.errhandler_arg= NULL;
+
+	argvec[0] = "--no-tty";
+	argvec[1]=NULL;
+	gi.argc=1;
+	gi.argv=argvec;
+
+	i=libmail_gpg_decode(LIBMAIL_GPG_UNENCRYPT|LIBMAIL_GPG_CHECKSIGN,
+			     &gi);
+	fclose(out_fp);
+	close(out_dup);
+	fclose(in_fp);
+	close(in_dup);
+	if (fp)
+		fclose(fp);
+
+	if (i)
 	{
-		printf("<blockquote><pre style=\"color: red;\">");
+		printf("<div class=\"indent\"><pre style=\"color: red;\">");
 		sent_gpgerrtxt();
-		printf("</pre></blockquote>\n");
+		printf("</pre></div>\n");
 	}
-	return (rc);
+	return (i);
 }
 
 int gpgexportkey(const char *fingerprint, int issecret,
@@ -725,10 +743,10 @@ int gpgexportkey(const char *fingerprint, int issecret,
 {
 	gpginiterr();
 
-	return (gpg_exportkey(GPGDIR, issecret, fingerprint,
-			      func,
-			      gpg_error,
-			      arg));
+	return (libmail_gpg_exportkey(GPGDIR, issecret, fingerprint,
+				      func,
+				      gpg_error,
+				      arg));
 }
 
 #else
