@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: html.c,v 1.15 2001/11/07 01:32:51 mrsam Exp $
+** $Id: html.c,v 1.20 2003/10/06 00:16:13 mrsam Exp $
 */
 #include	<stdio.h>
 #include	<ctype.h>
@@ -39,6 +39,7 @@ char	*q;
 			c=(unsigned char)c;
 			if (c)	*q++=c;
 			if (*p == ';')	p++;
+			continue;
 		}
 
 		for (i=1; p[i]; i++)
@@ -86,12 +87,14 @@ char	*q;
 
 	The ONLOAD, ONMOUSEOVER, and all other ON* attributes are removed.
 
-	Attributes TARGET, CODE, ACTION, CODETYPE and LANGUAGE are removed.
+	Attributes BACKGROUND, STYLE, TARGET, CODE, ACTION, CODETYPE and
+	LANGUAGE are removed.
 	TARGET=_blank is added to all <A> tags.
 
 	HREF, SRC, or LOWSRC attributes that do not specify a URL of http:,
 	https:, ftp:, gopher:, wais:, or telnet:, are removed.
 
+	Everything in <STYLE>   </STYLE> is dropped..
 */
 
 char *tagbuf=0;
@@ -214,6 +217,11 @@ char *p;
 
 static void parsetagbuf()
 {
+	char *p;
+
+	while ((p=strchr(tagbuf, '<')) != NULL)
+		*p=' ';
+
         tagattrlen=parseattr(0);
         if ( tagattrlen > tagattrsize)
         {
@@ -282,6 +290,10 @@ enum htmlstate {
 
 static enum htmlstate cur_state;
 static unsigned instyletag;
+static unsigned inscripttag;
+
+#define skipping() (instyletag || inscripttag)
+
 static void filtered_tag();
 static void (*htmlfiltered_func)(const char *, size_t);
 static char *(*htmlconvertcid_func)(const char *, void *);
@@ -296,6 +308,7 @@ void htmlfilter_init( void (*func)(const char *, size_t))
 	cur_state=intext;
 	htmlfiltered_func=func;
 	instyletag=0;
+	inscripttag=0;
 }
 
 /* Set prefix to wash HTML links */
@@ -330,8 +343,15 @@ size_t	start=0;
 	{
 		switch (cur_state)	{
 		case intext:
+			if (p[l] == '>')
+			{
+				(*htmlfiltered_func)(p+start, l-start);
+				(*htmlfiltered_func)("&nbsp;", 6);
+				start=l+1;
+			}
+
 			if (p[l] != '<')	continue;
-			if (!instyletag)
+			if (!skipping())
 				(*htmlfiltered_func)(p+start, l-start);
 					/* Output everything up until the tag */
 			cur_state=seenlt;
@@ -342,7 +362,7 @@ size_t	start=0;
 			{
 				cur_state=intext;
 				start=l+1;
-				if (!instyletag)
+				if (!skipping())
 					(*htmlfiltered_func)("<>", 2);
 						/* Eh? */
 				continue;
@@ -391,7 +411,7 @@ do_intag:
 			along */
 
 			if (p[l] != '-')	goto do_intag;
-			if (!instyletag)
+			if (!skipping())
 				(*htmlfiltered_func)("<!--", 4);
 			start=l+1;
 			cur_state=incomment;
@@ -413,7 +433,7 @@ do_intag:
 				cur_state=incomment;
 				continue;
 			}
-			if (!instyletag)
+			if (!skipping())
 				(*htmlfiltered_func)(p+start, l+1-start);
 			cur_state=intext;
 			start=l+1;
@@ -429,7 +449,7 @@ do_intag:
 	case incomment:
 	case incommentseendash:
 	case incommentseendashdash:
-		if (!instyletag)
+		if (!skipping())
 			(*htmlfiltered_func)(p+start, l-start);
 	default:
 		break;
@@ -560,8 +580,21 @@ int	open_window=0;
  
 	addtagbuf(0);
 	parsetagbuf();
+
+	if ( is_htmltag("SCRIPT"))
+	{
+		++inscripttag;
+		return;
+	}
+
+	if ( is_htmltag("/SCRIPT"))
+	{
+		if (inscripttag)
+			--inscripttag;
+		return;
+	}
+
 	if (	is_htmltag("TITLE") || is_htmltag("/TITLE") ||
-		is_htmltag("SCRIPT") || is_htmltag("/SCRIPT") ||
 		is_htmltag("FRAME") || is_htmltag("/FRAME") ||
 		is_htmltag("IFRAME") || is_htmltag("/IFRAME") ||
 		is_htmltag("APP") || is_htmltag("/APP") ||
@@ -584,11 +617,12 @@ int	open_window=0;
 
 	if ( is_htmltag("/STYLE"))
 	{
-		--instyletag;
+		if (instyletag)
+			--instyletag;
 		return;
 	}
 
-	if (instyletag)	return;
+	if (skipping())	return;
 
 
 	for (l=1; l<tagattrlen; l++)
@@ -660,7 +694,7 @@ int	open_window=0;
 				alt[tagattr[nalt].tagvaluelen]=0;
 			}
 
-			(*htmlfiltered_func)("<A TARGET=\"_blank\" HREF=\"", 25);
+			(*htmlfiltered_func)("<a target=\"_blank\" href=\"", 25);
 
 			if (washlink)	dowashlink(nsrc, q);
 			else	replacelink(nsrc, q);
@@ -676,7 +710,7 @@ int	open_window=0;
 				(*htmlfiltered_func)(q, strlen(q));
 				(*htmlfiltered_func)("]", 1);
 			}
-			(*htmlfiltered_func)("</A>", 4);
+			(*htmlfiltered_func)("</a>", 4);
 			free(q);
 			if (alt)	free(alt);
 		}
@@ -693,7 +727,10 @@ int	open_window=0;
 			is_attr(tagattr+l, "code") ||
 			is_attr(tagattr+l, "language") ||
 			is_attr(tagattr+l, "action") ||
-			is_attr(tagattr+l, "type") ||
+		    is_attr(tagattr+l, "background") ||
+		    is_attr(tagattr+l, "style") ||
+			(is_attr(tagattr+l, "type")
+			 && !is_htmltag("BLOCKQUOTE")) ||
 			is_attr(tagattr+l, "codetype"))
 			memset(tagattr[l].atagstart, ' ',
 				tagattr[l].ataglen);
