@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: newmsg.c,v 1.41 2001/08/22 01:04:08 mrsam Exp $
+** $Id: newmsg.c,v 1.54 2004/05/23 14:28:24 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -42,6 +42,9 @@
 #endif
 #include	<errno.h>
 #include	"htmllibdir.h"
+#if	HAVE_SQWEBMAIL_UNICODE
+#include	"unicode/unicode.h"
+#endif
 
 extern const char *sqwebmail_content_charset;
 extern int spell_start(const char *);
@@ -60,28 +63,89 @@ extern void output_urlencoded(const char *);
 extern char *newmsg_newdraft(const char *, const char *, const char *,
 				const char *);
 extern char *newmsg_createdraft(const char *);
-extern char *newmsg_createsentmsg(const char *, int *);
+extern char *newmsg_createsentmsg(const char *, int *, int);
 extern int ishttps();
 
-static void newmsg_header(const char *label, const char *field, const char *val)
+static void newmsg_header(const char *label, const char *field,
+			  const char *encoded, const char *val)
 {
-	printf("<TR><TH ALIGN=RIGHT><P CLASS=\"new-message-header\">"
-	       "<FONT CLASS=\"new-message-header-%s\">%s</FONT></TD>"
-	       "<TD WIDTH=6>&nbsp;</TD>",
+#if	HAVE_SQWEBMAIL_UNICODE
+const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
+#endif
+	printf("<tr><th align=\"right\"><p class=\"new-message-header\">"
+	       "<span class=\"new-message-header-%s\">%s</span></p></th>"
+	       "<td width=\"6\">&nbsp;</td>",
 	       field, label);
 
-	printf("<TD><INPUT NAME=%s SIZE=50 MAXLENGTH=512 VALUE=\"",
+	printf("<td><input name=\"%s\" size=\"50\" maxlength=\"512\" value=\"",
 		field);
-	if (val)
+	if (encoded)
 	{
 	char	*s;
 
-		s=rfc2047_decode_simple(val);
+#if	HAVE_SQWEBMAIL_UNICODE
+		s=rfc2047_decode_unicode(encoded, uiptr,
+			RFC2047_DECODE_NOTAG|RFC2047_DECODE_REPLACE);
+#else
+		s=rfc2047_decode_simple(encoded);
+#endif
 		if (!s)	enomem();
 		output_attrencoded(s);
 		free(s);
 	}
-	printf("\"></TD></TR>\n");
+	else if (val)
+		output_attrencoded(val);
+	printf("\" /></td></tr>\n");
+}
+
+static void printc(char c, void *dummy)
+{
+	char b[2];
+
+	b[0]=c;
+	b[1]=0;
+	output_attrencoded(b);
+}
+
+static void printsep(const char *c, void *dummy)
+{
+	output_attrencoded(c);
+}
+
+static void newmsg_header_rfc822(const char *label, const char *field,
+				 const char *encoded, const char *val)
+{
+	printf("<tr><th align=\"right\"><p class=\"new-message-header\">"
+	       "<span class=\"new-message-header-%s\">%s</span></p></th>"
+	       "<td width=\"6\">&nbsp;</td>",
+	       field, label);
+
+	printf("<td><input name=\"%s\" size=\"50\" maxlength=\"512\" value=\"",
+		field);
+	if (encoded)
+	{
+		struct rfc822t *t=rfc822t_alloc_new(encoded, NULL, NULL);
+		struct rfc822a *a=t ? rfc822a_alloc(t):NULL;
+
+		if (a)
+		{
+#if HAVE_SQWEBMAIL_UNICODE
+			rfc2047_print_unicode(a, sqwebmail_content_charset,
+					      printc,
+					      printsep, NULL);
+#else
+			rfc2047_print(a, sqwebmail_content_charset,
+				      printc, printsep, NULL);
+#endif
+			rfc822a_free(a);
+		}
+
+		if (t)
+			rfc822t_free(t);
+	}
+	else if (val)
+		output_attrencoded(val);
+	printf("\" /></td></tr>\n");
 }
 
 static const char *ispreviewmsg()
@@ -102,26 +166,39 @@ const char *p=cgi("previewmsg");
 
 void newmsg_hiddenheader(const char *label, const char *value)
 {
-	printf("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"", label);
+	printf("<input type=\"hidden\" name=\"%s\" value=\"", label);
 	output_attrencoded(value);
-	printf("\">");
+	printf("\" />");
 }
 
 /* ---------------------------------------------------- */
 
 /* Display message preview */
 
-static int filter_stub(const char *ptr, size_t cnt, void *voidptr)
-{
-        filter(ptr, cnt);
-        return (0);
-}
-
 static void preview_show_func_s(const char *p, size_t n)
 {
 	fwrite(p, 1, n, stdout);
 }
 
+void preview_start()
+{
+	printf("<pre class=\"message-text-plain\">");
+	filter_start(FILTER_FOR_PREVIEW, &preview_show_func_s);
+}
+
+int preview_callback(const char *ptr, size_t cnt, void *voidptr)
+{
+        filter(ptr, cnt);
+        return (0);
+}
+
+void preview_end()
+{
+	filter_end();
+	printf("</pre>\n");
+}
+
+#if 0
 static void show_preview(const char *filename)
 {
 char	*header, *value;
@@ -174,6 +251,7 @@ int	fd;
 	rfc2045_free(rfcp);
 	filter_end();
 }
+#endif
 
 static int show_textarea(const char *p, size_t l, void *voidptr)
 {
@@ -272,6 +350,50 @@ char *newmsg_alladdrs(FILE *fp)
 	return (p);
 }
 
+void newmsg_showfp(FILE *fp, int *attachcnt)
+{
+	struct	rfc2045 *p=rfc2045_fromfp(fp), *q;
+
+	if (!p)	enomem();
+
+	/* Here's a nice opportunity to count all attachments */
+
+	*attachcnt=0;
+
+	for (q=p->firstpart; q; q=q->next)
+		if (!q->isdummy)	++*attachcnt;
+	if (*attachcnt)	--*attachcnt;
+	/* Not counting the 1st MIME part */
+
+	q=rfc2045_searchcontenttype(p, "text/plain");
+
+	if (q)
+#if	HAVE_SQWEBMAIL_UNICODE
+		rfc2045_decodetextmimesection(fileno(fp), q,
+					      sqwebmail_content_charset,
+					      &show_textarea, NULL);
+#else
+		rfc2045_decodemimesection(fileno(fp), q,
+					  &show_textarea, NULL);
+#endif
+	rfc2045_free(p);
+}
+
+void newmsg_preview(const char *p)
+{
+	size_t pos;
+
+	maildir_remcache(INBOX "." DRAFTS);
+	if (maildir_name2pos(INBOX "." DRAFTS, p, &pos) == 0)
+	{
+		const char *save_folder=sqwebmail_folder;
+		cgi_put("showdraft", "1");
+		sqwebmail_folder=INBOX "." DRAFTS;
+		folder_showmsg(INBOX "." DRAFTS, pos);
+		sqwebmail_folder=save_folder;
+				/* show_preview(draftmessagefilename); */
+	}
+}
 
 /* ---------------------------------------------------- */
 
@@ -335,27 +457,35 @@ void newmsg_init(const char *folder, const char *pos)
 	}
 
 	draftmessagefilename= draftmessage ?
-				 maildir_find(DRAFTS, draftmessage):0;
+				 maildir_find(INBOX "." DRAFTS, draftmessage):0;
 
 	if (*(p=cgi("previewmsg")))
 	{
 #ifdef	ISPELL
 		if (strcmp(p, "SPELLCHK") == 0)
-			printf("%s<BR><BR>\n", checkspellingdone);
+			printf("%s<br /><br />\n", checkspellingdone);
 #endif
-		printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=1 BGCOLOR=\"#000000\" CLASS=\"box-small-outer\"><TR><TD>\n");
-		printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4 BGCOLOR=\"#FFFFFF\" CLASS=\"preview\"><TR><TD><TT><PRE>\n");
+		printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\" class=\"box-small-outer\"><tr><td>\n");
+		printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"4\" class=\"preview\"><tr><td>\n");
 
 		if (draftmessagefilename)
-			show_preview(draftmessagefilename);
+		{
+			const char *p=strrchr(draftmessagefilename, '/');
 
-		printf("</PRE></TT></TD></TR></TABLE>\n");
-		printf("</TD></TR></TABLE>\n");
+			if (p)
+				++p;
+			else
+				p=draftmessagefilename;
 
-		printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=6><TR><TD><HR WIDTH=\"80%%\"></TD></TR></TABLE>\n");
+			newmsg_preview(p);
+		}
+		printf("</td></tr></table>\n");
+		printf("</td></tr></table>\n");
+
+		printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"6\"><tr><td><hr width=\"80%%\" /></td></tr></table>\n");
 	}
 
-	printf("<INPUT TYPE=HIDDEN NAME=form VALUE=\"donewmsg\">\n");
+	printf("<input type=\"hidden\" name=\"form\" value=\"donewmsg\" />\n");
 	newmsg_hiddenheader("pos", pos);
 	newmsg_hiddenheader("focusto",
 			    *cgi("newmsg") ? "headers":"text");
@@ -368,9 +498,9 @@ void newmsg_init(const char *folder, const char *pos)
 
 	if (*cgi("foldermsg"))
 	{
-		printf("<P><FONT CLASS=\"error\" COLOR=\"#FF0000\">");
+		printf("<p><span class=\"error\" style=\"color: #ff0000\">");
 		output_attrencoded_nltobr(cgi("foldermsg"));
-		printf("</FONT><BR>");
+		printf("</span></p>");
 	}
 
 	if (strcmp(cgi("error"), "quota") == 0)
@@ -435,41 +565,43 @@ void newmsg_init(const char *folder, const char *pos)
 		}
 	}
 
-	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=1 BGCOLOR=\"#000000\" CLASS=\"box-small-outer\"><TR><TD>\n");
-	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4 BGCOLOR=\"#DDDDDD\" CLASS=\"new-message-box\"><TR><TD>\n");
+	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\" class=\"box-small-outer\"><tr><td>\n");
+	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"4\" class=\"new-message-box\"><tr><td>\n");
 
-	printf("<TABLE BORDER=0 WIDTH=\"100%%\">\n");
+	printf("<table border=\"0\" width=\"100%%\">\n");
 	if (access(NOCHANGINGFROM, 0))
-		newmsg_header(fromlab, "headerfrom", curfrom ? curfrom:
+		newmsg_header_rfc822(fromlab, "headerfrom", curfrom,
 			*cgi("from") ? cgi("from"):
 			pref_from && *pref_from ? pref_from:
 			login_fromhdr());
 
-	printf("<TR VALIGN=MIDDLE><TH ALIGN=RIGHT>"
-	       "<P CLASS=\"new-message-header\">"
-	       "<FONT CLASS=\"new-message-header-addressbook\">"
-	       "%s</FONT></TD><TD WIDTH=6>&nbsp;</TH>",
+	printf("<tr valign=\"middle\"><th align=\"right\">"
+	       "<p class=\"new-message-header\">"
+	       "<span class=\"new-message-header-addressbook\">"
+	       "%s</span></p></th><td width=\"6\">&nbsp;</td>",
 	       addressbooklab);
 
-	printf("<TD VALIGN=MIDDLE>");
-	printf("<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=4>");
-	printf("<TR VALIGN=MIDDLE><TD><SELECT NAME=\"nick\" SIZE=4 MULTIPLE>\n");
+	printf("<td valign=\"middle\">");
+	printf("<table border=\"0\" cellpadding=\"0\" cellspacing=\"4\">");
+	printf("<tr valign=\"middle\"><td><select name=\"nick\" size=\"4\" multiple=\"multiple\">\n");
 	ab_listselect();
-	printf("</select></TD><TD WIDTH=\"100%%\">");
-	printf("<input type=submit name=\"addressbook_to\" value=\"%s\">",
+	printf("</select></td><td width=\"100%%\">");
+	printf("<input type=\"submit\" name=\"addressbook_to\" value=\"%s\" />",
 			tolab);
-	printf("<input type=submit name=\"addressbook_cc\" value=\"%s\">",
+	printf("<input type=\"submit\" name=\"addressbook_cc\" value=\"%s\" />",
 			cclab);
-	printf("<input type=submit name=\"addressbook_bcc\" value=\"%s\">",
+	printf("<input type=\"submit\" name=\"addressbook_bcc\" value=\"%s\" />",
 			bcclab);
-	printf("</TD></TR></TABLE>");
+	printf("</td></tr></table>");
 
-	printf("</TD></TR>\n");
-	newmsg_header(tolab, "headerto", curto ? curto:cgi("to"));
-	newmsg_header(cclab, "headercc", curcc ? curcc:cgi("cc"));
-	newmsg_header(bcclab, "headerbcc", curbcc ? curbcc:cgi("bcc"));
-	newmsg_header(replytolab, "headerreply-to", curreplyto ? curreplyto:cgi("replyto"));
-	newmsg_header(subjectlab, "headersubject", cursubj ? cursubj:cgi("subject"));
+	printf("</td></tr>\n");
+	newmsg_header_rfc822(tolab, "headerto", curto, cgi("to"));
+	newmsg_header_rfc822(cclab, "headercc", curcc, cgi("cc"));
+	newmsg_header_rfc822(bcclab, "headerbcc", curbcc, cgi("bcc"));
+	newmsg_header_rfc822(replytolab, "headerreply-to",
+			     curreplyto, cgi("replyto"));
+	newmsg_header(subjectlab, "headersubject",
+		      cursubj, cgi("subject"));
 
 	if (curto)	free(curto);
 	if (curfrom)	free(curfrom);
@@ -478,33 +610,18 @@ void newmsg_init(const char *folder, const char *pos)
 	if (curbcc)	free(curbcc);
 	if (cursubj)	free(cursubj);
 
-	printf("<TR><TD COLSPAN=3><HR WIDTH=\"100%%\"></TD></TR>");
-	printf("<TR><TH VALIGN=TOP ALIGN=RIGHT>"
-	       "<P CLASS=\"new-message-header\">"
-	       "<FONT CLASS=\"new-message-header-message\">"
-	       "%s</FONT></P></TD><TD WIDTH=6>&nbsp;</TD>", messagelab);
+	printf("<tr><td colspan=\"3\"><hr width=\"100%%\" /></td></tr>");
+	printf("<tr><th valign=\"top\" align=\"right\">"
+	       "<p class=\"new-message-header\">"
+	       "<span class=\"new-message-header-message\">"
+	       "%s</span></p></th><td width=\"6\">&nbsp;</td>", messagelab);
 
-	printf("<TD><TEXTAREA NAME=message COLS=%d ROWS=15 WRAP=soft>",
+	printf("<td><textarea name=\"message\" cols=\"%d\" rows=\"15\" wrap=\"soft\">",
 		MYLINESIZE);
 
 	if (fp)
 	{
-	struct	rfc2045 *p=rfc2045_fromfp(fp), *q;
-
-		/* Here's a nice opportunity to count all attachments */
-
-		for (q=p->firstpart; q; q=q->next)
-			if (!q->isdummy)	++attachcnt;
-		if (attachcnt)	--attachcnt;
-			/* Not counting the 1st MIME part */
-		if (!p)	enomem();
-
-		q=rfc2045_searchcontenttype(p, "text/plain");
-
-		if (q)
-			rfc2045_decodemimesection(fileno(fp), q,
-						  &show_textarea, NULL);
-		rfc2045_free(p);
+		newmsg_showfp(fp, &attachcnt);
 	}
 	else
 	{
@@ -524,49 +641,55 @@ void newmsg_init(const char *folder, const char *pos)
 			fp=NULL;
 		}
 	}
-	printf("</TEXTAREA><BR>\n");
+	printf("</textarea><br />\n");
 
 	if (draftmessage && *draftmessage)
 	{
-		printf("<INPUT TYPE=HIDDEN NAME=draftmessage VALUE=\"");
+		printf("<input type=\"hidden\" name=\"draftmessage\" value=\"");
 		output_attrencoded(draftmessage);
-		printf("\">");
+		printf("\" />");
 	}
 	if (draftmessage)	free(draftmessage);
-	printf("</TD></TR>\n");
+	printf("</td></tr>\n");
 
-	printf("<TR><TD><P>&nbsp</TD><TD>&nbsp;</TD><TD><INPUT TYPE=SUBMIT NAME=doattachments VALUE=\"");
+	printf("<tr><td><p>&nbsp;</p></td><td>&nbsp;</td><td><input type=\"submit\" name=\"doattachments\" value=\"");
 
 	printf(uploadlab, attachcnt);
-	printf("\"></TD></TR>");
+	printf("\" /></td></tr>");
 
-	printf("<TR><TD COLSPAN=2 ALIGN=RIGHT><INPUT TYPE=CHECKBOX "
-	       "NAME=fcc%s></TD><TD>%s</TD></TR>\n",
-	       pref_noarchive ? "":" CHECKED",
+	printf("<tr><td colspan=\"2\" align=\"right\"><input type=\"checkbox\" "
+	       "name=\"fcc\" id=\"fcc\"%s /></td><td><label for=\"fcc\">%s</label></td></tr>\n",
+	       pref_noarchive ? "":" checked=\"checked\"",
 	       getarg("PRESERVELAB"));
+
+	if (access(NODSN, R_OK))
+		printf("<tr><td colspan=\"2\" align=\"right\"><input type=\"checkbox\" "
+		       "name=\"dsn\" id=\"dsn\" /></td><td><label for=\"dsn\">%s</label></td></tr>\n",
+		       getarg("DSN"));
+
 #if HAVE_SQWEBMAIL_UNICODE
-	if (has_gpg(GPGDIR) == 0)
+	if (libmail_gpg_has_gpg(GPGDIR) == 0)
 	{
 		char *all_addr;
 
-		printf("<TR><TD COLSPAN=2 ALIGN=RIGHT><INPUT TYPE=CHECKBOX "
-		       "NAME=sign></TD><TD>%s<SELECT NAME=signkey>",
+		printf("<tr><td colspan=\"2\" align=\"right\"><input type=\"checkbox\" "
+		       "name=\"sign\" id=\"sign\" /></td><td><label for=\"sign\">%s</label><select name=\"signkey\">",
 		       getarg("SIGNLAB"));
 		gpgselectkey();
-		printf("</SELECT></TD></TR>\n");
+		printf("</select></td></tr>\n");
 
 		all_addr=newmsg_alladdrs(fp);
 
-		printf("<TR VALIGN=MIDDLE><TD COLSPAN=2 ALIGN=RIGHT>"
-		       "<INPUT TYPE=CHECKBOX NAME=encrypt></TD>"
-		       "<TD><TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 VALIGN=MIDDLE><TR><TD>%s</TD><TD><SELECT SIZE=4 MULTIPLE NAME=encryptkey>",
+		printf("<tr valign=\"middle\"><td colspan=\"2\" align=\"right\">"
+		       "<input type=\"checkbox\" name=\"encrypt\" id=\"encrypt\" /></td>"
+		       "<td><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr valign=\"middle\"><td><label for=\"encrypt\">%s</label></td><td><select size=\"4\" multiple=\"multiple\" name=\"encryptkey\">",
 		       getarg("ENCRYPTLAB"));
 		gpgencryptkeys(all_addr);
-		printf("</SELECT></TD></TR>\n");
-		printf("</TABLE></TD></TR>\n");
+		printf("</select></td></tr>\n");
+		printf("</table></td></tr>\n");
 
 		if (ishttps())
-			printf("<TR><TD COLSPAN=2 ALIGN>&nbsp;</TD><TD>%s<input type=password name=passphrase></TD></TR>\n",
+			printf("<tr><td colspan=\"2\" align=\"left\">&nbsp;</td><td>%s<input type=\"password\" name=\"passphrase\" /></td></tr>\n",
 			       getarg("PASSPHRASE"));
 
 		if (all_addr)
@@ -579,22 +702,22 @@ void newmsg_init(const char *folder, const char *pos)
 	if (draftmessagefilename)
 		free(draftmessagefilename);
 
-	printf("<TR><TD COLSPAN=2>&nbsp;</TD><TD>");
-	printf("<INPUT TYPE=SUBMIT NAME=previewmsg VALUE=\"%s\">",
+	printf("<tr><td colspan=\"2\">&nbsp;</td><td>");
+	printf("<input type=\"submit\" name=\"previewmsg\" value=\"%s\" />",
 		previewlab);
-	printf("<INPUT TYPE=SUBMIT NAME=sendmsg VALUE=\"%s\">",
+	printf("<input type=\"submit\" name=\"sendmsg\" value=\"%s\" />",
 		sendlab);
-	printf("<INPUT TYPE=SUBMIT NAME=savedraft VALUE=\"%s\">",
+	printf("<input type=\"submit\" name=\"savedraft\" value=\"%s\" />",
 		savedraft);
 #ifdef	ISPELL
-	printf("<INPUT TYPE=SUBMIT NAME=startspellchk VALUE=\"%s\">",
+	printf("<input type=\"submit\" name=\"startspellchk\" value=\"%s\" />",
 		checkspelling);
 #endif
-	printf("</TD></TR>\n");
-	printf("</TD></TABLE>\n");
+	printf("</td></tr>\n");
+	printf("</table>\n");
 
-	printf("</TD></TR></TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
+	printf("</td></tr></table>\n");
+	printf("</td></tr></table>\n");
 }
 
 static const char *geterrbuf(int fd)
@@ -643,6 +766,9 @@ char	*filename;
 const char *line;
 char	*draftmessage;
 int	isgpgerr;
+unsigned long filesize;
+struct stat stat_buf;
+int dsn;
 
 	if (tokencheck()) /* Duplicate submission - message was already sent */
 	{
@@ -662,7 +788,7 @@ int	isgpgerr;
 	if (!draftmessage)
 		enomem();
 
-	filename=newmsg_createsentmsg(draftmessage, &isgpgerr);
+	filename=newmsg_createsentmsg(draftmessage, &isgpgerr, 1);
 
 	if (!filename)
 	{
@@ -687,13 +813,13 @@ int	isgpgerr;
 	if (pipe(pipefd1) != 0)
 	{
 		cgi_put("foldermsg", "ERROR: pipe() failed.");
-		maildir_msgpurgefile(SENT, filename);
+		maildir_msgpurgefile(INBOX "." SENT, filename);
 		free(filename);
 		free(draftmessage);
 		return (0);
 	}
 
-	returnaddr=login_returnaddr();
+	dsn= *cgi("dsn") != 0;
 
 	pid=fork();
 	if (pid < 0)
@@ -701,7 +827,7 @@ int	isgpgerr;
 		cgi_put("foldermsg", "ERROR: fork() failed.");
 		close(pipefd1[0]);
 		close(pipefd1[1]);
-		maildir_msgpurgefile(SENT, filename);
+		maildir_msgpurgefile(INBOX "." SENT, filename);
 		free(filename);
 		free(draftmessage);
 		return (0);
@@ -711,7 +837,7 @@ int	isgpgerr;
 	{
 	static const char noexec[]="ERROR: Unable to execute sendit.sh.\n";
 	static const char nofile[]="ERROR: Temp file not available - probably exceeded quota.\n";
-	char	*tmpfile=maildir_find(SENT, filename);
+	char	*tmpfile=maildir_find(INBOX "." SENT, filename);
 	int	fd;
 
 		if (!tmpfile)
@@ -730,10 +856,18 @@ int	isgpgerr;
 		close(pipefd1[0]);
 		close(pipefd1[1]);
 
+		if (dsn)
+			putenv("DSN=-Nsuccess,delay,fail");
+		else
+			putenv("DSN=");
+
 		if (fd == 0)
+		{
+			returnaddr=login_returnaddr();
 			execl(SENDITSH, "sendit.sh", returnaddr,
 				sqwebmail_mailboxid, NULL);
-
+		}
+			
 		fwrite(noexec, 1, sizeof(noexec)-1, stderr);
 		_exit(1);
 	}
@@ -755,7 +889,7 @@ int	isgpgerr;
 		if (*draftmessage)
 		{
 		char	*base=maildir_basename(draftmessage);
-		char	*draftfile=maildir_find(DRAFTS, base);
+		char	*draftfile=maildir_find(INBOX "." DRAFTS, base);
 
 			free(base);
 
@@ -767,17 +901,11 @@ int	isgpgerr;
 			char	*header, *value;
 			FILE	*fp;
 			int	x;
-			unsigned long filesize;
-			int	rc;
-			int	quotafd;
-			char	quotabuf[QUOTABUFSIZE];
 
 				fp=0;
 				x=maildir_safeopen(draftfile, O_RDONLY, 0);
 				if ( maildir_parsequota(draftfile, &filesize))
 				{
-				struct stat stat_buf;
-
 					if (x < 0 || fstat(x, &stat_buf))
 						stat_buf.st_size=0;
 					filesize=stat_buf.st_size;
@@ -818,31 +946,8 @@ int	isgpgerr;
 				if (replytofolder)	free(replytofolder);
 				if (replytomsg)	free(replytomsg);
 				
-				if (maildir_getquota(".", quotabuf))
-				{
-					if (errno != ENOENT)    enomem();
-					quotabuf[0]=0;
-				}
-
-				do
-				{
-					if (quotabuf[0] == 0)	break;
-
-					rc=maildir_checkquota(".", &quotafd,
-						quotabuf, -filesize, -1);
-					if (rc && errno != EAGAIN)
-					{
-						if (quotafd >= 0)
-							close(quotafd);
-						enomem();
-					}
-				} while (rc);
-				if (quotabuf[0])
-				{
-					maildir_addquota(".", quotafd, quotabuf,
-						-filesize, -1);
-					if (quotafd >= 0)	close(quotafd);
-				}
+				maildir_quota_deleted(".",
+						      -(long)filesize, -1);
 
 				unlink(draftfile);
 				free(draftfile);
@@ -853,40 +958,14 @@ int	isgpgerr;
 
 		if (*cgi("fcc") == 0)
 		{
-			int quotafd;
-			int rc;
-			char quotabuf[QUOTABUFSIZE];
 			unsigned long filesize=0;
-			char	*tmpfile=maildir_find(SENT, filename);
+			char	*tmpfile=maildir_find(INBOX "." SENT, filename);
 
 			if (tmpfile)
 			{
-				maildir_parsequota(filename, &filesize);
+				maildir_parsequota(tmpfile, &filesize);
 				unlink(tmpfile);
-
-				if (maildir_getquota(".", quotabuf) == 0 &&
-				    quotabuf[0])
-				{
-					do
-					{
-						rc=maildir_checkquota(".",
-								      &quotafd,
-								      quotabuf,
-								      -(long)filesize,
-								      -1);
-						if (rc && errno != EAGAIN)
-							break;
-					} while (rc);
-
-					if (rc == 0)
-						maildir_addquota(".",
-								 quotafd,
-								 quotabuf,
-								 -(long)filesize,
-								 -1);
-					if (quotafd >= 0)
-						close(quotafd);
-				}
+				maildir_quota_deleted(".", -(long)filesize,-1);
 				free(tmpfile);
 			}
 		}
@@ -896,7 +975,10 @@ int	isgpgerr;
 		sendmsg_done();
 		return (1);
 	}
-	maildir_msgpurgefile(SENT, filename);
+
+	if (stat(filename, &stat_buf) == 0)
+		maildir_quota_deleted(".", -(long)stat_buf.st_size, -1);
+	maildir_msgpurgefile(INBOX "." SENT, filename);
 	free(filename);
 
 	{
