@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: sqispell.c,v 1.19 2001/12/07 12:54:59 mrsam Exp $
+** $Id: sqispell.c,v 1.21 2004/01/25 05:38:50 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"maildir.h"
@@ -31,7 +31,7 @@ static int search_spell(const char *, unsigned, unsigned);
 
 int spell_start(const char *c)
 {
-char	*filename=maildir_find(DRAFTS, c);
+char	*filename=maildir_find(INBOX "." DRAFTS, c);
 
 	if (!c)	return (-1);
 
@@ -414,34 +414,34 @@ const char *finishlab=getarg("FINISH");
 
 	CHECKFILENAME(draftmessage);
 
-	printf("<INPUT TYPE=HIDDEN NAME=form VALUE=\"spellchk\">\n");
-	printf("<INPUT TYPE=HIDDEN NAME=pos VALUE=\"%s\">\n", cgi("pos"));
+	printf("<input type=\"hidden\" name=\"form\" value=\"spellchk\" />\n");
+	printf("<input type=\"hidden\" name=\"pos\" value=\"%s\" />\n", cgi("pos"));
 	if (*cgi("globignore"))
 	{
-		printf("<INPUT TYPE=HIDDEN NAME=globignore VALUE=\"");
+		printf("<input type=\"hidden\" name=\"globignore\" value=\"");
 		output_attrencoded(cgi("globignore"));
-		printf("\">\n");
+		printf("\" />\n");
 	}
 	if (*cgi("globreplace"))
 	{
-		printf("<INPUT TYPE=HIDDEN NAME=globreplace VALUE=\"");
+		printf("<input type=\"hidden\" name=\"globreplace\" value=\"");
 		output_attrencoded(cgi("globreplace"));
-		printf("\">\n");
+		printf("\" />\n");
 	}
 
-	printf("<INPUT TYPE=HIDDEN NAME=draftmessage VALUE=\"");
+	printf("<input type=\"hidden\" name=\"draftmessage\" value=\"");
 	output_attrencoded(draftmessage);
-	printf("\">");
-	printf("<INPUT TYPE=HIDDEN NAME=row VALUE=\"%u\"><INPUT TYPE=HIDDEN NAME=col VALUE=\"%u\"><INPUT TYPE=HIDDEN NAME=word VALUE=\"",
+	printf("\" />");
+	printf("<input type=\"hidden\" name=\"row\" value=\"%u\" /><input type=\"hidden\" name=\"col\" value=\"%u\" /><input type=\"hidden\" name=\"word\" value=\"",
 		(unsigned)paragraph,
 		(unsigned)msp->word_pos);
 	output_attrencoded(msp->misspelled_word);
 
-	printf("\"><TABLE BORDER=0 CELLSPACING=0 CELLPADDING=1 "
-	       "CLASS=\"box-small-outer\"><TR><TD>");
-	printf("<TABLE BORDER=0 CELLSPACING=0 CLASS=\"spellcheck-background\"><TR><TD>");
+	printf("\" /><table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" "
+	       "class=\"box-small-outer\"><tr><td>");
+	printf("<table border=\"0\" cellspacing=\"0\" class=\"spellcheck-background\"><tr><td>");
 
-	printf("<TABLE BORDER=1 CELLSPACING=0 CELLPADDING=8 CLASS=\"spellcheck-excerpt\"><TR><TD ALIGN=CENTER><TT><FONT COLOR=\"#000000\" class=\"spellcheck-excerpt\">");
+	printf("<table border=\"1\" cellspacing=\"0\" cellpadding=\"8\" class=\"spellcheck-excerpt\"><tr><td align=\"center\"><span style=\"color: #000000\" class=\"spellcheck-excerpt\">");
 
 	if (msp->word_pos > 30)
 	{
@@ -461,11 +461,11 @@ const char *finishlab=getarg("FINISH");
 	filter_start(FILTER_FOR_DISPLAY, &showfunc);
 	filter(ispellline+p, msp->word_pos-p);
 	filter_end();
-	printf("<B>");
+	printf("<strong>");
 	filter_start(FILTER_FOR_DISPLAY, &showfunc);
 	filter(ispellline+msp->word_pos, strlen(msp->misspelled_word));
 	filter_end();
-	printf("</B>");
+	printf("</strong>");
 	p=msp->word_pos+strlen(msp->misspelled_word);
 	if (l-p < 30)
 	{
@@ -489,40 +489,40 @@ const char *finishlab=getarg("FINISH");
 	filter_end();
 	if (n != l-p)
 		printf("&nbsp;...");
-	printf("</FONT></TT></TD></TR></TABLE><BR>");
-	printf("<TABLE BORDER=1 CELLPADDING=8 CLASS=\"spellcheck-main\"><TR><TD>");
+	printf("</span></td></tr></table><br />");
+	printf("<table border=\"1\" cellpadding=\"8\" class=\"spellcheck-main\"><tr><td>");
 
-	printf("<TABLE BORDER=0><TR><TD>");
+	printf("<table border=\"0\">");
 	for (isps=msp->first_suggestion; isps; isps=isps->next)
 	{
-		printf("<TR><TD>%s</TD><TD><INPUT BORDER=0 TYPE=RADIO NAME=REPLACE VALUE=\"%s\"></TD><TD>%s</TD></TR>\n",
+		printf("<tr><td>%s</td><td><input type=\"radio\" name=\"REPLACE\" value=\"%s\" /></td><td>%s</td></tr>\n",
 			replacelab,
 			isps->suggested_word,
 			isps->suggested_word);
 		replacelab=" ";
 	}
-	printf("<TR><TD>%s</TD><TD><INPUT BORDER=0 TYPE=RADIO NAME=REPLACE VALUE=\"#other\"></TD><TD><INPUT TYPE=TEXT NAME=OTHER WIDTH=20></TD></TR>\n",
+	printf("<tr><td>%s</td><td><input type=\"radio\" name=\"REPLACE\" value=\"#other\" /></td><td><input type=\"text\" name=\"OTHER\" width=\"20\" /></td></tr>\n",
 		replacelab);
-	printf("<TR><TD> </TD><TD><INPUT BORDER=0 TYPE=RADIO NAME=REPLACE VALUE=\"#insert\"></TD><TD>%s</TD></TR>\n",
+	printf("<tr><td> </td><td><input type=\"radio\" name=\"REPLACE\" value=\"#insert\" /></td><td>%s</td></tr>\n",
 		insertlab);
 
-	printf("<TR><TD> </TD><TD><INPUT BORDER=0 TYPE=CHECKBOX NAME=REPLACEALL></TD><TD>%s</TD></TR>\n",
+	printf("<tr><td> </td><td><input type=\"checkbox\" name=\"REPLACEALL\" /></td><td>%s</td></tr>\n",
 		replacealllab);
-	printf("<TR><TD> </TD><TD COLSPAN=2><HR WIDTH=\"100%%\"></TD></TR>\n");
-	printf("<TR><TD> </TD><TD><INPUT BORDER=0 TYPE=RADIO NAME=REPLACE VALUE=\"#ignore\"></TD><TD>%s</TD></TR>\n",
+	printf("<tr><td> </td><td colspan=\"2\"><hr width=\"100%%\" /></td></tr>\n");
+	printf("<tr><td> </td><td><input type=\"radio\" name=\"REPLACE\" value=\"#ignore\" /></td><td>%s</td></tr>\n",
 		ignorelab);
-	printf("<TR><TD> </TD><TD><INPUT BORDER=0 TYPE=RADIO NAME=REPLACE VALUE=\"#ignoreall\"></TD><TD>%s</TD></TR>\n",
+	printf("<tr><td> </td><td><input type=\"radio\" name=\"REPLACE\" value=\"#ignoreall\" /></td><td>%s</td></tr>\n",
 		ignorealllab);
-	printf("</TABLE>");
-	printf("</TD></TR></TABLE><BR>");
-	printf("<TABLE BORDER=1 CELLPADDING=8 CLASS=\"spellcheck-continue\"><TR><TD>");
-	printf("<INPUT TYPE=SUBMIT NAME=\"continue\" VALUE=\"%s\">\n",
+	printf("</table>");
+	printf("</td></tr></table><br />");
+	printf("<table border=\"1\" cellpadding=\"8\" class=\"spellcheck-continue\"><tr><td>");
+	printf("<input type=\"submit\" name=\"continue\" value=\"%s\" />\n",
 			continuelab);
-	printf("<INPUT TYPE=SUBMIT NAME=\"finish\" VALUE=\"%s\">\n",
+	printf("<input type=\"submit\" name=\"finish\" value=\"%s\" />\n",
 			finishlab);
-	printf("</TD></TR></TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
+	printf("</td></tr></table>\n");
+	printf("</td></tr></table>\n");
+	printf("</td></tr></table>\n");
 }
 
 static FILE *opendict(const char *mode)
@@ -617,7 +617,7 @@ unsigned pos=atol(cgi("col"));
 char	*draftfilename;
 
 	CHECKFILENAME(filename);
-	draftfilename=maildir_find(DRAFTS, filename);
+	draftfilename=maildir_find(INBOX "." DRAFTS, filename);
 	if (!draftfilename)
 	{
 		output_form("folder.html");
